#ifndef _graph_bcode_function_h_
#define _graph_bcode_function_h_

// (C) by Thalion Graphic Solutions e.K. 2001
//
// Permissions to copy, modify and redistribute
// granted, according to the conditions defined
// in the Gnu Public License (GPL).
// Please read the file gpl.txt provided with this
// distribution.

#include <graph/graph3d.h>
#include <graph/runtime.h>

#include <string.h>

//
// The classes defined below allow graphing of
// functions defined in bytecode.
//

namespace graph {

   template <int GW,int GH>
   class BCodeFunction2:
      public GraphableFunction2<GW,GH> {

      public:
         //
	 // Errors.
	 //
	 class typemismatch_error {};

         //
	 // Methods.
	 //	 
         BCodeFunction2(Runtime&,const Function&);
         BCodeFunction2& operator=(const Function&);
         virtual double operator () (double,double);

      private:
         const Function *f_;
         Runtime        *env_;
   };

   template <int GW,int GH>
   class BCodeFunction3:
      public GraphableFunction3<GW,GH> {

      public:
         //
	 // Errors.
	 //
	 class typemismatch_error {};
	 
	 //
	 // Methods.
	 //
         BCodeFunction3(Runtime&,const Function&);
         BCodeFunction3& operator=(const Function&);
         virtual double operator () (double,double,double);

      private:
         const Function *f_;
         Runtime        *env_;

   };

}

template <int GW,int GH>
inline graph::BCodeFunction2<GW,GH>::BCodeFunction2(Runtime& env,
                                                    const Function& f):
   env_(&env),f_(&f)
{
   if ( strcmp(f_->getType(),"(ff)f") ) {
      throw typemismatch_error();
   }
}

template <int GW,int GH>
inline graph::BCodeFunction2<GW,GH>& graph::BCodeFunction2<GW,GH>::operator=(const graph::Function& rhs)
{
   if ( !strcmp(rhs.getType(),"(ff)f") ) {
      f_ = &rhs;
   }
   else {
      throw typemismatch_error();
   }
}

template <int GW,int GH>
inline graph::BCodeFunction3<GW,GH>::BCodeFunction3(Runtime& env,
                                                    const Function& f):
   env_(&env),f_(&f)
{
   if ( strcmp(f_->getType(),"(fff)f") ) {
      throw typemismatch_error();
   }
}

template <int GW,int GH>
inline graph::BCodeFunction3<GW,GH>& graph::BCodeFunction3<GW,GH>::operator=(const graph::Function& rhs)
{
   if ( !strcmp(rhs.getType(),"(fff)f") ) {
      f_ = &rhs;
   }
   else {
      throw typemismatch_error();
   }
}

#endif
