#ifndef _graph_refcounted_h_
#define _graph_refcounted_h_

// (C) by Thalion Graphic Solutions e.K. 2001
//
// Permissions to copy, modify and redistribute
// granted, according to the conditions defined
// in the Gnu Public License (GPL).
// Please read the file gpl.txt provided with this
// distribution.

namespace graph {

   class RefCounted {
   
      public:
         RefCounted();

	 void ref();
	 void unref();

      protected:
         virtual ~RefCounted();

      private:
         // Reference counted objects should be handled
	 // via references - no immediate construction of
	 // copies should be required. Use the clone feature
	 // instead.
	 RefCounted(RefCounted&);
	 RefCounted(const RefCounted&);
	 RefCounted& operator=(RefCounted&);
	 RefCounted& operator=(const RefCounted&);
	 
	 long ref_;
   
   };
   
}

inline void graph::RefCounted::ref()
{
   ref_++;
}

inline void graph::RefCounted::unref()
{
   ref_--;
   if ( ! ref_ ) {
      delete this;
   }
}

#endif
