package graph3d;

// (C) by Thalion Graphic Solutions e.K. 2001
//
// Permissions to copy, modify and redistribute
// granted, according to the conditions defined
// in the Gnu Public License (GPL).
// Please read the file gpl.txt provided with this
// distribution.

import java.awt.*;

/**
 * This GraphicsConfigTemplate will select an OpenGL suitable
 * configuration (see the native implementation for details).
 */
public class GraphicsConfigTemplate
   extends java.awt.GraphicsConfigTemplate {

   /**
    * Use this constant to indicate, that
    * you want to create a config template for the
    * viewport config.
    */
   public static final int VIEWPORT = 1;

   /**
    * Use this constant, to find the
    * offscreen config used for printing and
    * file saving.
    */
   public static final int OFFSCREEN = 2;

   private int type_;

   /**
    * The method checks the passed configuration, and returns true,
    * if it is a suitable one.
    * @param gc a configuration to check
    * @return true, if it's usable, else false
    */
   private boolean checkConfiguration(GraphicsConfiguration gc)
   {
      switch ( type_ ) {
         case VIEWPORT:
            return graph3d.peer.GraphicsConfigTemplate.checkVpConfiguration(gc);
         case OFFSCREEN:
            return graph3d.peer.GraphicsConfigTemplate.checkOffsConfiguration(gc);
         default:
            return false;
      }
   }

   /**
    * Find the optimal configuration.
    * Here, we use a simplicistic approach: we consider the first
    * match to be the 'optimal' one.
    */
   public GraphicsConfiguration getBestConfiguration(GraphicsConfiguration[] gc)
   {
      for ( int i=0; i<gc.length; i++ ) {
         if ( checkConfiguration(gc[i]) ) {
            return gc[i];
         }
      }
      return null;
   }
   
   /**
    * @param gc a configuration to be checked
    * @return true, if it is supported, else false.
    */
   public boolean isGraphicsConfigSupported(GraphicsConfiguration gc)
   {
      return checkConfiguration(gc);   
   }   

   /**
    * Create a new template for one of the
    * types VIEWPORT or OFFSCREEN.
    * @param type VIEWPORT or OFFSCREEN
    */
   public GraphicsConfigTemplate(int type)
   {
      if ( type == VIEWPORT || type == OFFSCREEN ) {
         type_ = type;
      }
      else {
         throw new IllegalArgumentException("type");
      }
   }
   
}
