package ui;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JToolBar;

import java.awt.event.*;
import java.awt.*;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.Reader;
import java.io.StringReader;

import graph3d.CodeGenerator;
import graph3d.Function;
import graph3d.Function2;
import graph3d.Function3;
import graph3d.GraphView;
import graph3d.RuntimeException;
import graph3d.TypeMismatchException;

import mathc.Lexer;
import mathc.ParserContext;
import mathc.TypeChecker;

public class App extends JFrame {

    private static App instance_;
    
    static {
       instance_ = new App();
    }
    
    private ParserContext parser_;
    private TypeChecker   checker_;
    
    private JToolBar      doctools_;
    private JToolBar      apptools_;
    private JDesktopPane  desktop_;
    private JFileChooser  chooser_;

    public static App getInstance()
    {
       return instance_;
    }
    
    public App() {
        super("JGraph");

        //Make the big window be indented 50 pixels from each edge 
        //of the screen.
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	int width = ( screenSize.width > 800 ? 800 : screenSize.width);
	int height = (screenSize.height > 600 ? 600 : screenSize.height);
        setBounds((screenSize.width - width) / 2,
                  (screenSize.height - height) / 2,
                  width, height);

        //Quit this app when the big window closes.
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        //Set up the GUI.
	getContentPane().setLayout(new BorderLayout());
        desktop_ = new JDesktopPane(); //a specialized layered pane
	apptools_ = createAppTools();
	doctools_ = new JToolBar(JToolBar.VERTICAL);
	getContentPane().add(apptools_,BorderLayout.NORTH);
        getContentPane().add(doctools_,BorderLayout.WEST);
        getContentPane().add(desktop_,BorderLayout.CENTER);
        setJMenuBar(createMenuBar());

        //Make dragging faster:
        desktop_.putClientProperty("JDesktopPane.dragMode", "outline");
	
	chooser_ = new JFileChooser();
	
	parser_      = new ParserContext();
	checker_ = new TypeChecker(parser_);
    }

    protected JToolBar createAppTools()
    {
       JToolBar toolbar_ = new JToolBar(JToolBar.HORIZONTAL);
       JPanel panel = new JPanel();
       panel.setLayout(new FlowLayout(FlowLayout.LEFT));
       JButton b = new JButton();
       ResourceManager.setIconForButton("new",b);
       b.setToolTipText(ResourceManager.getToolTip("define-function"));
       b.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent event)
	  {
	     InputDialog.showDialog();
          }
       });
       panel.add(b);
       b = new JButton();
       ResourceManager.setIconForButton("open",b);
       b.setToolTipText(ResourceManager.getToolTip("open"));
       b.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent event)
	  {
             int returnVal = chooser_.showOpenDialog(App.getInstance());
             if(returnVal == JFileChooser.APPROVE_OPTION) {
                openFileCB(chooser_.getSelectedFile());
             }  
          }
       });
       panel.add(b);
       b = new JButton();
       ResourceManager.setIconForButton("plot",b);
       b.setToolTipText(ResourceManager.getToolTip("plot-function"));
       b.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent event)
	  {
             ListDialog.showDialog();
          }
       });
       panel.add(b);
       b = new JButton();
       ResourceManager.setIconForButton("reset",b);
       b.setToolTipText(ResourceManager.getToolTip("reset"));
       b.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent event)
	  {
             resetCB();
          }
       });
       panel.add(b);
       b = new JButton();
       ResourceManager.setIconForButton("preferences",b);
       b.setToolTipText(ResourceManager.getToolTip("edit-preferences"));
       b.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent event)
	  {
             PreferencesDialog.showDialog();
          }
       });
       panel.add(b);
       toolbar_.add(panel);
       return toolbar_;
    }

    protected JMenuBar createMenuBar()
    {
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);

        JMenuBar menuBar = new JMenuBar();

        JMenu menu = new JMenu(ResourceManager.getMenuTitle("file"));
        menu.setMnemonic(KeyEvent.VK_F);
        JMenuItem menuItem = new JMenuItem(ResourceManager.getMenuTitle("new"));
        menuItem.setMnemonic(KeyEvent.VK_N);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                InputDialog.showDialog();
            }
        });
        menu.add(menuItem);
        menuItem = new JMenuItem(ResourceManager.getMenuTitle("open"));
	menuItem.setMnemonic(KeyEvent.VK_O);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
               int returnVal = chooser_.showOpenDialog(App.getInstance());
               if(returnVal == JFileChooser.APPROVE_OPTION) {
                  openFileCB(chooser_.getSelectedFile());
               }    
            }
        });
        menu.add(menuItem);
        menuItem = new JMenuItem(ResourceManager.getMenuTitle("plot"));
	menuItem.setMnemonic(KeyEvent.VK_P);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
               ListDialog.showDialog();
            }
        });
        menu.add(menuItem);
        menuItem = new JMenuItem(ResourceManager.getMenuTitle("reset"));
	menuItem.setMnemonic(KeyEvent.VK_R);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
               resetCB();
            }
        });
        menu.add(menuItem);	
	menu.add(new JSeparator());
        menuItem = new JMenuItem(ResourceManager.getMenuTitle("exit"));
	menuItem.setMnemonic(KeyEvent.VK_X);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exitCB();
            }
        });
        menu.add(menuItem);

	menuBar.add(menu);

        menu = new JMenu(ResourceManager.getMenuTitle("edit"));
        menu.setMnemonic(KeyEvent.VK_E);

        menuItem = new JMenuItem(ResourceManager.getMenuTitle("preferences"));
        menuItem.setMnemonic(KeyEvent.VK_P);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                PreferencesDialog.showDialog();
            }
        });
        menu.add(menuItem);

	menuBar.add(menu);

        menu = new JMenu(ResourceManager.getMenuTitle("help"));
        menu.setMnemonic(KeyEvent.VK_H);

        menuItem = new JMenuItem(ResourceManager.getMenuTitle("about"));
        menuItem.setMnemonic(KeyEvent.VK_A);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Info.showAboutDialog();
            }
        });
        menu.add(menuItem);

	menuBar.add(menu);

	return menuBar;
    }
    
    private void updateListDialog()
    {
       ListDialog.updateDialog(CodeGenerator.getInstance().getFunctions().toArray());
    }
    
    private void parseDefinitions(Reader in)
    {
       try {
          parser_.setReader(in);
          parser_.parse();
          checker_.check();
	  checker_.optimize();
	  CodeGenerator.getInstance().flush();
          checker_.emitCode(CodeGenerator.getInstance());
          updateListDialog();
       }
       catch ( Lexer.Exception exception ) {
          JOptionPane.showMessageDialog(this,
                                        exception.getMessage(),
                                        ResourceManager.getErrorMessage("syntax-error"),
                                        JOptionPane.ERROR_MESSAGE);
       }
       catch ( ParserContext.Exception exception ) {
          JOptionPane.showMessageDialog(this,
                                        exception.getMessage(),
                                        ResourceManager.getErrorMessage("syntax-error"),
                                        JOptionPane.ERROR_MESSAGE);
       }
       catch ( TypeChecker.CheckErrors exception ) {
          JOptionPane.showMessageDialog(this,
                                        exception.getMessage(),
                                        ResourceManager.getErrorMessage("typecheck"),
                                        JOptionPane.ERROR_MESSAGE);
       }
       catch ( CodeGenerator.DataSizeExceededException exception ) {
          JOptionPane.showMessageDialog(this,
                                        exception.getMessage(),
                                        ResourceManager.getErrorMessage("dbuffer"),
                                        JOptionPane.ERROR_MESSAGE);       
       }
       catch ( Exception exception ) {
          JOptionPane.showMessageDialog(this,
                                        exception.getMessage(),
                                        ResourceManager.getErrorMessage("error"),
                                        JOptionPane.ERROR_MESSAGE);
          exception.printStackTrace();
       }    
    }
    
    public void newCB(String text)
    {
       parseDefinitions(new StringReader(text));
    }
    
    protected void openFileCB(File file)
    {
       try {
          parseDefinitions(new FileReader(file));
       }
       catch ( FileNotFoundException exception ) {
          JOptionPane.showMessageDialog(this,
                                        exception.getMessage(),
                                        ResourceManager.getErrorMessage("file-not-found"),
                                        JOptionPane.ERROR_MESSAGE);      
       }
    }
    
    public void plotCB(Function f)
    {
       createDocument(f);
    }
    
    public void resetCB()
    {
       if ( JOptionPane.showConfirmDialog(this,
                                          ResourceManager.getDialogString("reset-jgraph"),
                                          ResourceManager.getDialogString("confirm"),
					  JOptionPane.YES_NO_OPTION,
                                          JOptionPane.QUESTION_MESSAGE,
					  null) == JOptionPane.YES_OPTION ) {
          JInternalFrame[] frames = desktop_.getAllFrames();
	  // Close all document windows.
	  for ( int i=0; i<frames.length; i++ ) {
	     frames[i].dispose();
	  }
	  // Reset the code generator.
	  CodeGenerator.getInstance().flush();
	  // We need to reallocate those, because they recall and rewrite all
	  // definitions parsed so far.
	  parser_      = new ParserContext();
	  checker_ = new TypeChecker(parser_);	  				
          updateListDialog();
       }       
    }
    
    protected void exitCB()
    {
       if ( JOptionPane.showConfirmDialog(this,
                                          ResourceManager.getDialogString("exit-jgraph"),
                                          ResourceManager.getDialogString("confirm"),
					  JOptionPane.YES_NO_OPTION,
                                          JOptionPane.QUESTION_MESSAGE,
					  null) == JOptionPane.YES_OPTION ) {
          Preferences.getInstance().save();
          System.exit(0);
       }
    }

    protected void createDocument(Function function) {
        try {
           DocumentWindow dw = new DocumentWindow(function);
           dw.setVisible(true); //necessary as of kestrel
           desktop_.add(dw);
            dw.setSelected(true);
        }
	catch ( IllegalArgumentException exception ) {
          JOptionPane.showMessageDialog(this,
                                        function.toString(),
                                        ResourceManager.getErrorMessage("invalid-range"),
                                        JOptionPane.ERROR_MESSAGE);	
	}
	catch ( RuntimeException exception ) {
          JOptionPane.showMessageDialog(this,
                                        function.toString() + ": " + exception.getMessage(),
                                        ResourceManager.getErrorMessage("graph-runtime"),
                                        JOptionPane.ERROR_MESSAGE);	
	}
	catch ( TypeMismatchException exception ) {
          JOptionPane.showMessageDialog(this,
                                        function.toString(),
                                        ResourceManager.getErrorMessage("function-not-plottable"),
                                        JOptionPane.ERROR_MESSAGE);	
	}
	catch (java.beans.PropertyVetoException e) {}
    }

    public static void main(String[] args)
    {
        Info.showStartupLogo(5);
        App.getInstance().setVisible(true);
        try {
	   for ( int i=0; i<args.length; i++ ) {
	      App.getInstance().parseDefinitions(new FileReader(args[i]));
	   }
        }
        catch ( FileNotFoundException exception ) {
           JOptionPane.showMessageDialog(App.getInstance(),
                                         exception.getMessage(),
                                         ResourceManager.getErrorMessage("file-not-found"),
                                         JOptionPane.ERROR_MESSAGE);      
        }
    }
    
    public void setTools(DocumentTools tools)
    {
       doctools_.removeAll();
       if ( tools != null ) {
          tools.setToolBar(doctools_);
       }
       doctools_.repaint();
    }

}
