package ui;

import javax.swing.AbstractButton;
import javax.swing.ImageIcon;

import java.awt.MediaTracker;

import java.util.Hashtable;

/**
 * This class serves as a factory for all
 * UI objects which require access to icons.
 */
class IconFactory {

   public final static int DEFAULT =  0;
   public final static int PRESSED =  1;
   public final static int ROLLOVER = 2;

   private final static String[] suffices_ = {
      "-default.gif",
      "-pressed.gif",
      "-rollover.gif"
   };

   private static Hashtable hash_;

   /**
    * Load one set of icons (used as one symbol on a
    * control - different named images mark up different states).
    * @param name the icon set name
    * @return an array of image icons, representing the set
    */
   private static ImageIcon[] loadIconSet(String name)
   {
      ImageIcon[] set = new ImageIcon[3];
      for( int i=0; i<3; i++ ) {
         String imagename = "images/" + name + suffices_[i];
         set[i] = new ImageIcon(imagename);
         if ( set[i].getImageLoadStatus() != MediaTracker.COMPLETE ) {
            System.err.println("Failed to load '" + imagename + "'.");
         }
      }
      return set;
   }

   /**
    * Preloading all icons.
    */
   static {
      hash_ = new Hashtable();
      hash_.put("axes",loadIconSet("axes"));      
      hash_.put("contours",loadIconSet("contours"));
      hash_.put("define",loadIconSet("define"));
      hash_.put("define-all",loadIconSet("define-all"));
      hash_.put("light0",loadIconSet("light0"));
      hash_.put("light1",loadIconSet("light1"));
      hash_.put("lighting",loadIconSet("lighting"));
      hash_.put("lines",loadIconSet("lines"));
      hash_.put("loop",loadIconSet("loop"));
      hash_.put("new",loadIconSet("new"));
      hash_.put("none",loadIconSet("none"));
      hash_.put("open",loadIconSet("open"));
      hash_.put("plot",loadIconSet("plot"));
      hash_.put("points",loadIconSet("points"));
      hash_.put("printer",loadIconSet("printer"));
      hash_.put("reset",loadIconSet("reset"));
      hash_.put("smooth",loadIconSet("smooth"));
      hash_.put("swing",loadIconSet("swing"));
   }

   /**
    * Get the named icon for a given state.
    * @param name the icon (set) name
    * @param state the state
    * @return the icon
    */
   public static ImageIcon getIcon(String name,int state)
   {
      return ((ImageIcon[])hash_.get(name))[state];
   }

   /**
    * Set the icon (set) of a given button.
    * @param name the icon (set) name
    * @param b the button
    */
   public static void setIconForButton(String name,AbstractButton b)
   {
      b.setIcon(getIcon(name,DEFAULT));
      b.setPressedIcon(getIcon(name,PRESSED));
      b.setSelectedIcon(getIcon(name,PRESSED));
      b.setRolloverIcon(getIcon(name,ROLLOVER));
      b.setRolloverSelectedIcon(getIcon(name,ROLLOVER));
      b.setRolloverEnabled(true);
      b.setContentAreaFilled(false);
      b.setBorderPainted(false);
   }

}
