package ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;

import java.io.IOException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JWindow;

/**
 * The class implements a logo popup,
 * which will be shown at program startup.
 */

class Info {

   private static Info instance_;

   static {
      instance_ = new Info();
   }

   private JPanel      logo_;
   private JScrollPane about_;

   Info()
   {
      JLabel imagelabel = new JLabel(ResourceManager.getIcon("logo"));
      JLabel versionlabel =
         new JLabel("Release " + Version.getVersionString(),JLabel.RIGHT);
      logo_ = new JPanel();
      logo_.setLayout(new BorderLayout());
      logo_.add(imagelabel,BorderLayout.CENTER);
      logo_.add(versionlabel,BorderLayout.SOUTH);
      JEditorPane about = new JEditorPane();
      about.setEditable(false);
      String s = null;
      try {
         URL url = new URL(ResourceManager.getString("info-about-url"));
         about.setPage(url);
      }
      catch ( Exception e ) {
         System.err.println(ResourceManager.getErrorMessage("file-not-found") + ": " + s);
      }
      about_ = new JScrollPane(about);
      about_.setPreferredSize(new Dimension(400, 80));

   }

   JPanel getLogo()
   {
      return logo_;
   }

   JScrollPane getAbout()
   {
      return about_;
   }

   static Info getInstance()
   {
      return instance_;
   }

   static void showAboutDialog()
   {
      final JDialog d = new JDialog(App.getInstance());
      d.getContentPane().setLayout(new BorderLayout());
      d.getContentPane().add(getInstance().getLogo(),BorderLayout.NORTH);
      d.getContentPane().add(getInstance().getAbout(),BorderLayout.CENTER);
      JPanel panel = new JPanel();
      JButton button = new JButton();
      ResourceManager.setIconForButton("none",button);
      button.setToolTipText(ResourceManager.getToolTip("close"));
      button.addActionListener(
         new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
               d.setVisible(false);
            }
         }
      ); 
      panel.setLayout(new FlowLayout());
      panel.add(button);
      d.getContentPane().add(panel,BorderLayout.SOUTH);
      d.setModal(true);
      d.setResizable(false);
      d.pack();
      d.setVisible(true);
   }

   /**
    * Shows the logo as a startup window.
    * After a given delay, the window will
    * be made invisible.
    * The method blocks, until the delay time
    * has been elapsed.
    */
   static void showStartupLogo(int delay)
   {
      JWindow w = new JWindow();
      w.getContentPane().add(getInstance().getLogo());
      w.pack();
      Dimension screensize = Toolkit.getDefaultToolkit().getScreenSize();
      int x = (screensize.width - w.getWidth()) / 2;
      int y = (screensize.height - w.getHeight()) / 2;
      w.setLocation(x,y);
      w.setVisible(true);
      w.toFront();    
      try {
         Thread.sleep(delay * 1000);
      }
      catch ( Exception e ) {}
      w.setVisible(false);
   }

   static void main(String[] argv)
   {
      Info.showStartupLogo(5);
      Info.showAboutDialog();
   }

}
