package ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import graph3d.Function;

/**
 * Copied and modified from the Java tutorial (file of same name).
 * Modified: the dialog takes and returns indices instead of strings.
 */
public class ListDialog extends JDialog {

    private static ListDialog dialog_;

    static {
        Frame frame = JOptionPane.getFrameForComponent(App.getInstance());
        dialog_     = new ListDialog(frame,
                                     ResourceManager.getDialogString("plot"),
                                     ResourceManager.getDialogString("defined-functions"));    
    }

    private JList             list_;
    
    public static void showDialog()
    {
       dialog_.setVisible(true);
    }
    
    public static void updateDialog(Object[] values)
    {
       dialog_.setItems(values);
       dialog_.repaint();
    }
    
    public void setValue(int index) {
        list_.setSelectedIndex(index);
    }
    
    private void setItems(Object[] values)
    {
       list_.setListData(values);
    }

    private ListDialog(Frame frame, String title, String labelText) {
        super(frame, title, false);

        //buttons
        JButton cb = new JButton();
        ResourceManager.setIconForButton("none",cb);
        cb.setToolTipText(ResourceManager.getToolTip("close"));
        cb.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ListDialog.dialog_.setVisible(false);
            }
        });
        final JButton sb = new JButton();
        ResourceManager.setIconForButton("plot",sb);
        sb.setToolTipText(ResourceManager.getToolTip("plot-it"));
        sb.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                App.getInstance().plotCB((Function) list_.getSelectedValue());
            }
        });
        getRootPane().setDefaultButton(sb);

        //main part of the dialog
        list_ = new JList();
        list_.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        list_.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    sb.doClick();
                }
            }
        });
        JScrollPane listScroller = new JScrollPane(list_);
        listScroller.setPreferredSize(new Dimension(250, 160));
        //XXX: Must do the following, too, or else the scroller thinks
        //XXX: it's taller than it is:
        listScroller.setMinimumSize(new Dimension(250, 160));
        listScroller.setAlignmentX(LEFT_ALIGNMENT);

        //Create a container so that we can add a title around
        //the scroll pane.  Can't add a title directly to the 
        //scroll pane because its background would be white.
        //Lay out the label and scroll pane from top to button.
        JPanel listPane = new JPanel();
        listPane.setLayout(new BoxLayout(listPane, BoxLayout.Y_AXIS));
        JLabel label = new JLabel(labelText);
        label.setLabelFor(list_);
        listPane.add(label);
        listPane.add(Box.createRigidArea(new Dimension(0,5)));
        listPane.add(listScroller);
        listPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

        //Lay out the buttons from left to right.
        JPanel buttonPane = new JPanel();
        buttonPane.add(cb);
        //buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
        buttonPane.add(sb);

        //Put everything together, using the content pane's BorderLayout.
        Container contentPane = getContentPane();
        contentPane.add(listPane, BorderLayout.CENTER);
        contentPane.add(buttonPane, BorderLayout.SOUTH);

        pack();
    }

}
