package ui;

import java.awt.Color;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.Serializable;
import java.io.StreamCorruptedException;

import graph3d.GraphView;

class Preferences
   implements Serializable {

   private static Preferences instance_;

   static {
      try {
         ObjectInputStream is = new ObjectInputStream
            (new FileInputStream(ResourceManager.getString("preferences-file")));
         instance_ = (Preferences) is.readObject();
      }
      catch ( ClassNotFoundException exception ) {
         instance_ = new Preferences();
      }
      catch ( OptionalDataException exception ) {
         instance_ = new Preferences();
      }
      catch ( StreamCorruptedException exception ) {
         instance_ = new Preferences();
      }
      catch ( IOException exception ) {
         instance_ = new Preferences();
      }
   }

   /**
    * MathC settings.
    */
   private boolean mathc_optimize_;

   /**
    * Graph properties.
    */
   private int     graph_modebits_;

   private Color[] graph_light_color_;
   private Color   graph_bg_color_;

   private double  graph_x0_;
   private double  graph_x1_;
   private double  graph_y0_;
   private double  graph_y1_;
   private double  graph_z0_;
   private double  graph_z1_;

   /**
    * Create a default set of preferences.
    */
   Preferences()
   {
      mathc_optimize_       = false;
      graph_modebits_       = GraphView.LINES;
      graph_light_color_    = new Color[2];
      graph_light_color_[0] = Color.red;
      graph_light_color_[1] = Color.blue;
      graph_bg_color_       = Color.black;
      graph_x0_             = -1;
      graph_x1_             =  1;
      graph_y0_             = -1;
      graph_y1_             =  1;
      graph_z0_             = -1;
      graph_z1_             =  1;
   }

   static Preferences getInstance()
   {
      return instance_;
   }

   void save()
   {
      try {
         ObjectOutputStream os = new ObjectOutputStream
            (new FileOutputStream(ResourceManager.getString("preferences-file")));
         os.writeObject(this);
      }
      catch ( Exception exception ) {
         System.err.println("Could not write preferences to `" +
                            ResourceManager.getString("preferences-file") + ')');
      }
   }

   boolean getMathcOptimize()
   {
      return mathc_optimize_;
   }

   int getGraphModeBits()
   {
      return graph_modebits_;
   }

   Color getGraphLightColor(int n)
   {
      return graph_light_color_[n];
   }

   Color getGraphBackgroundColor()
   {
      return graph_bg_color_;
   }


   double getGraphRangeX0()
   {
      return graph_x0_;
   }

   double getGraphRangeX1()
   {
      return graph_x1_;
   }

   double getGraphRangeY0()
   {
      return graph_y0_;
   }

   double getGraphRangeY1()
   {
      return graph_y1_;
   }

   double getGraphRangeZ0()
   {
      return graph_z0_;
   }

   double getGraphRangeZ1()
   {
      return graph_z1_;
   }

   void setMathcOptimize(boolean flag)
   {
      mathc_optimize_ = flag;
   }

   void setGraphModeBits(int bits)
   {
      graph_modebits_ = bits;
   }

   void setGraphLightColor(int n,Color c)
   {
      graph_light_color_[n] = c;
   }

   void setGraphBackgroundColor(Color c)
   {
      graph_bg_color_ = c;
   }


   void setGraphRangeX(double x0,double x1)
   {
      graph_x0_ = x0;
      graph_x1_ = x1;
   }

   void setGraphRangeY(double y0,double y1)
   {
      graph_y0_ = y0;
      graph_y1_ = y1;
   }

   void setGraphRangeZ(double z0,double z1)
   {
      graph_z0_ = z0;
      graph_z1_ = z1;
   }

}
