package ui;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import graph3d.GraphView;

/**
 * 
 */
public class PreferencesDialog extends JDialog {

    private static PreferencesDialog dialog_;

    static {
        Frame frame = JOptionPane.getFrameForComponent(App.getInstance());
        dialog_     = new PreferencesDialog(frame,
                                            ResourceManager.getDialogString("preferences"),
                                            ResourceManager.getDialogString("available-options"));    
    }

    private JCheckBox mathc_optimize_;

    private JCheckBox  graph_axes_;
    private JCheckBox  graph_contours_;
    private JCheckBox  graph_lighting_;
    private JCheckBox  graph_dlists_;
    private JCheckBox  graph_points_;
    private JCheckBox  graph_lines_;
    private JCheckBox  graph_smooth_;

    private JButton    graph_light0_color_;
    private JButton    graph_light1_color_;
    private JButton    graph_bg_color_;

    private DecimalField graph_x0_;
    private DecimalField graph_x1_;
    private DecimalField graph_y0_;
    private DecimalField graph_y1_;
    private DecimalField graph_z0_;
    private DecimalField graph_z1_;

    private JPanel createMathcSettingsTab()
    {
       mathc_optimize_ = new JCheckBox(ResourceManager.getLabelString("optimize"));
       mathc_optimize_.setSelected(Preferences.getInstance().getMathcOptimize());
       JPanel panel = new JPanel();
       //panel.setLayout();
       panel.add(mathc_optimize_);
       return panel;
    }

    private JPanel createGraphSettingsTab()
    {
       graph_axes_     = new JCheckBox(ResourceManager.getLabelString("axes"));
       graph_contours_ = new JCheckBox(ResourceManager.getLabelString("contours"));
       graph_lighting_ = new JCheckBox(ResourceManager.getLabelString("lighting"));
       graph_dlists_   = new JCheckBox(ResourceManager.getLabelString("dlists"));
       graph_points_   = new JCheckBox(ResourceManager.getLabelString("points"));
       graph_lines_    = new JCheckBox(ResourceManager.getLabelString("lines"));
       graph_smooth_   = new JCheckBox(ResourceManager.getLabelString("smooth"));
       ButtonGroup buttongroup = new ButtonGroup();
       buttongroup.add(graph_points_);
       buttongroup.add(graph_lines_);
       buttongroup.add(graph_smooth_);
       graph_light0_color_ = new JButton(ResourceManager.getLabelString("light0"));
       graph_light1_color_ = new JButton(ResourceManager.getLabelString("light1"));
       graph_bg_color_ = new JButton(ResourceManager.getLabelString("background"));
       graph_axes_.setToolTipText(ResourceManager.getToolTip("draw-axes"));
       graph_contours_.setToolTipText(ResourceManager.getToolTip("use-contours"));
       graph_lighting_.setToolTipText(ResourceManager.getToolTip("use-lighting"));
       graph_dlists_.setToolTipText(ResourceManager.getToolTip("use-lighting"));
       graph_points_.setToolTipText(ResourceManager.getToolTip("use-dlists"));
       graph_lines_.setToolTipText(ResourceManager.getToolTip("draw-lines"));
       graph_smooth_.setToolTipText(ResourceManager.getToolTip("draw-smooth"));
       graph_light0_color_.setToolTipText(ResourceManager.getToolTip("edit-light0"));
       graph_light1_color_.setToolTipText(ResourceManager.getToolTip("edit-light1"));
       graph_bg_color_.setToolTipText(ResourceManager.getToolTip("edit-background"));
       graph_axes_.setSelected((Preferences.getInstance().getGraphModeBits() & GraphView.AXES) != 0);
       graph_contours_.setSelected((Preferences.getInstance().getGraphModeBits() & GraphView.CONTOURS) != 0);
       graph_lighting_.setSelected((Preferences.getInstance().getGraphModeBits() & GraphView.LIGHTS) != 0);
       graph_dlists_.setSelected((Preferences.getInstance().getGraphModeBits() & GraphView.DLISTS) != 0);
       graph_points_.setSelected((Preferences.getInstance().getGraphModeBits() & GraphView.POINTS) != 0);
       graph_lines_.setSelected((Preferences.getInstance().getGraphModeBits() & GraphView.LINES) != 0);
       graph_smooth_.setSelected((Preferences.getInstance().getGraphModeBits() & GraphView.SMOOTH) != 0);
       graph_light0_color_.setBackground(Preferences.getInstance().getGraphLightColor(0));
       graph_light1_color_.setBackground(Preferences.getInstance().getGraphLightColor(1));
       graph_bg_color_.setBackground(Preferences.getInstance().getGraphBackgroundColor());
       graph_light0_color_.addActionListener(
          new ActionListener() {
             public void actionPerformed(ActionEvent event)
             {
                Color c = JColorChooser.showDialog(dialog_,
                                                   ResourceManager.getDialogString("set-light0"),
                                                   graph_light0_color_.getBackground());
                if ( c != null ) {
		   graph_light0_color_.setBackground(c);
		}
             }
          }
       );
       graph_light1_color_.addActionListener(
          new ActionListener() {
             public void actionPerformed(ActionEvent event)
             {
                Color c = JColorChooser.showDialog(dialog_,
                                                   ResourceManager.getDialogString("set-light1"),
                                                   graph_light1_color_.getBackground());
                if ( c != null ) {
		   graph_light1_color_.setBackground(c);
		}
             }
          }
       );
       graph_bg_color_.addActionListener(
          new ActionListener() {
             public void actionPerformed(ActionEvent event)
             {
                Color c = JColorChooser.showDialog(dialog_,
                                                   ResourceManager.getDialogString("set-bg"),
                                                   graph_bg_color_.getBackground());
		if ( c != null ) {				   
                   graph_bg_color_.setBackground(c);
		}
             }
          }
       );
       JPanel panel = new JPanel();
       //panel.setLayout();
       JPanel modepanel = new JPanel();
       modepanel.setLayout(new GridLayout(3,1));
       modepanel.add(graph_axes_);
       modepanel.add(graph_contours_);
       modepanel.add(graph_lighting_);
       modepanel.add(graph_dlists_);
       JPanel shadepanel = new JPanel();
       shadepanel.setLayout(new GridLayout(3,1));
       shadepanel.add(graph_points_);
       shadepanel.add(graph_lines_);
       shadepanel.add(graph_smooth_);
       JPanel colorpanel = new JPanel();
       colorpanel.setLayout(new GridLayout(3,1));
       colorpanel.add(graph_light0_color_);
       colorpanel.add(graph_light1_color_);
       colorpanel.add(graph_bg_color_);
       panel.add(modepanel);
       panel.add(shadepanel);
       panel.add(colorpanel);
       return panel;
    }

    private JPanel createGraphRangesTab()
    {
       graph_x0_ = new DecimalField(Preferences.getInstance().getGraphRangeX0(),10);
       graph_x1_ = new DecimalField(Preferences.getInstance().getGraphRangeX1(),10);
       graph_y0_ = new DecimalField(Preferences.getInstance().getGraphRangeY0(),10);
       graph_y1_ = new DecimalField(Preferences.getInstance().getGraphRangeY1(),10);
       graph_z0_ = new DecimalField(Preferences.getInstance().getGraphRangeZ0(),10);
       graph_z1_ = new DecimalField(Preferences.getInstance().getGraphRangeZ1(),10);
       JPanel xpanel = new JPanel();
       xpanel.add(graph_x0_);
       xpanel.add(new JLabel("-"));
       xpanel.add(graph_x1_);
       JPanel ypanel = new JPanel();
       ypanel.add(graph_y0_);
       ypanel.add(new JLabel("-"));
       ypanel.add(graph_y1_);
       JPanel zpanel = new JPanel();
       zpanel.add(graph_z0_);
       zpanel.add(new JLabel("-"));
       zpanel.add(graph_z1_);
       JPanel panel = new JPanel();
       panel.setLayout(new GridLayout(3,1));
       panel.add(xpanel);
       panel.add(ypanel);
       panel.add(zpanel);
       return panel;
    }

    public static void showDialog()
    {
       dialog_.setVisible(true);
    }

    private PreferencesDialog(Frame frame, String title, String labelText) {
        super(frame, title, false);

        //buttons
        JButton cb = new JButton();
        ResourceManager.setIconForButton("none",cb);
        cb.setToolTipText(ResourceManager.getToolTip("close"));
        final JButton ab = new JButton();
        ResourceManager.setIconForButton("define",ab);
        ab.setToolTipText(ResourceManager.getToolTip("apply-settings"));

        cb.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                PreferencesDialog.dialog_.setVisible(false);
            }
        });
        ab.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Preferences.getInstance().setMathcOptimize(mathc_optimize_.isSelected());
                int modebits = 0;
                if ( graph_axes_.isSelected() ) {
                   modebits |= GraphView.AXES;
                }
                if ( graph_contours_.isSelected() ) {
                   modebits |= GraphView.CONTOURS;
                }
                if ( graph_lighting_.isSelected() ) {
                   modebits |= GraphView.LIGHTS;
                }
                if ( graph_dlists_.isSelected() ) {
                   modebits |= GraphView.DLISTS;
                }
                if ( graph_points_.isSelected() ) {
                   modebits |= GraphView.POINTS;
                }
                if ( graph_lines_.isSelected() ) {
                   modebits |= GraphView.LINES;
                }
                if ( graph_smooth_.isSelected() ) {
                   modebits |= GraphView.SMOOTH;
                }
                Preferences.getInstance().setGraphModeBits(modebits);
                Preferences.getInstance().setGraphLightColor(0,graph_light0_color_.getBackground());
                Preferences.getInstance().setGraphLightColor(1,graph_light1_color_.getBackground());
                Preferences.getInstance().setGraphBackgroundColor(graph_bg_color_.getBackground());
                Preferences.getInstance().setGraphRangeX(graph_x0_.getValue(),graph_x1_.getValue());
                Preferences.getInstance().setGraphRangeY(graph_y0_.getValue(),graph_y1_.getValue());
                Preferences.getInstance().setGraphRangeZ(graph_z0_.getValue(),graph_z1_.getValue());
            }
        });
        getRootPane().setDefaultButton(ab);

        // Create tabs.
        JTabbedPane tabs = new JTabbedPane();
        tabs.addTab(ResourceManager.getTabString("graph-settings"),
                    null,
                    createGraphSettingsTab(),
                    ResourceManager.getToolTip("edit-graph-settings"));
        tabs.addTab(ResourceManager.getTabString("graph-ranges"),
                    null,
                    createGraphRangesTab(),
                    ResourceManager.getToolTip("edit-graph-ranges"));
        tabs.addTab(ResourceManager.getTabString("mathc-settings"),
                    null,
                    createMathcSettingsTab(),
                    ResourceManager.getToolTip("edit-mathc-settings"));

        //Lay out the buttons from left to right.
        JPanel buttons = new JPanel();
        buttons.add(cb);
        buttons.add(ab);

        //Put everything together, using the content pane's BorderLayout.
        Container contentPane = getContentPane();
        contentPane.add(tabs, BorderLayout.CENTER);
        contentPane.add(buttons, BorderLayout.SOUTH);

        pack();
    }

}
