package ui;

import javax.swing.AbstractButton;
import javax.swing.ImageIcon;

import java.awt.MediaTracker;

import java.util.Hashtable;
import java.util.MissingResourceException;
import java.util.ResourceBundle;


/**
 * This class serves as a factory for all
 * UI objects which require access to icons.
 */
class ResourceManager {

   public final static int DEFAULT =  0;
   public final static int PRESSED =  1;
   public final static int ROLLOVER = 2;

   private static ResourceBundle images_;
   private static ResourceBundle strings_;

   /**
    * Selects the resource bundle.
    */
   static {
      try {
         images_ = ResourceBundle.getBundle("ui.res.Images");
         strings_ = ResourceBundle.getBundle("ui.res.Strings");	
      }
      catch ( MissingResourceException e ) {
         System.err.println("Could not find resources.");
      }
   }

   /**
    * Get the named icon for a given state.
    * @param name the icon (set) name
    * @param state the state
    * @return the icon
    */
   public static ImageIcon getIcon(String name,int state)
   {
      return ((ImageIcon[])images_.getObject(name))[state];
   }

   /**
    * Get an icon not belonging to a set.
    * @param name the icon name
    * @return the icon
    */
   public static ImageIcon getIcon(String name)
   {
      return (ImageIcon)images_.getObject(name);
   }

   /**
    * Set the icon (set) of a given button.
    * @param name the icon (set) name
    * @param b the button
    */
   public static void setIconForButton(String name,AbstractButton b)
   {
      b.setIcon(getIcon(name,DEFAULT));
      b.setPressedIcon(getIcon(name,PRESSED));
      b.setSelectedIcon(getIcon(name,PRESSED));
      b.setRolloverIcon(getIcon(name,ROLLOVER));
      b.setRolloverSelectedIcon(getIcon(name,ROLLOVER));
      b.setRolloverEnabled(true);
      b.setContentAreaFilled(false);
      b.setBorderPainted(false);
   }

   /**
    * Get a string resource.
    * If no match is found, the key string will be returned.
    * @param key the resources' key
    * @return the resource, if any - else the key itself will
    *         be returned
    */
   public static String getString(String key)
   {
      String result = strings_.getString(key);
      if ( result != null ) {
         return result;
      }
      else {
         System.err.println("String resource '" + key + "' not found.");
         return key;
      }
   }

   public static String getDialogString(String key)
   {
      return getString("dialog-" + key);
   }

   public static String getErrorMessage(String key)
   {
      return getString("errmsg-" + key);
   }

   public static String getLabelString(String key)
   {
      return getString("label-" + key);
   }

   public static String getMenuTitle(String key)
   {
      return getString("menu-" + key);
   }

   public static String getTabString(String key)
   {
      return getString("tab-" + key);
   }

   public static String getToolTip(String key)
   {
      return getString("tooltip-" + key);
   }

}
