// (C) by Thalion Graphic Solutions e.K. 2001
//
// Permissions to copy, modify and redistribute
// granted, according to the conditions defined
// in the Gnu Public License (GPL).
// Please read the file gpl.txt provided with this
// distribution.

#include "jgraph.h"
#include "trace.h"
#include <assert.h>

#include <graph/function.h>


//////////////////////////////////////////////////////////////////////////////////
//
// Non-public members of namespace graph:
// contour plot color functions.
//
namespace graph {

   class RFunc: public graph::Function3 {
      public:
         static Function3& getInstance();
         double operator () (double,double,double);
         graph::Function3 *clone() const;
      private:
         static Function3 *instance_;
   };

   class GFunc: public graph::Function3 {
      public:
         static Function3& getInstance();
         double operator () (double,double,double);
         graph::Function3 *clone() const;
      private:
         static Function3 *instance_;
   };

   class BFunc: public graph::Function3 {
      public:
         static Function3& getInstance();
         double operator () (double,double,double);
         graph::Function3 *clone() const;
      private:
         static Function3 *instance_;
   };

}

graph::Function3 *graph::RFunc::instance_ = NULL;

graph::Function3& graph::RFunc::getInstance()
{
   trace("graph::RFunc::getInstance");
   if ( ! instance_ ) {
      instance_ = new RFunc();
      instance_->ref();
   }
   return *instance_;
}

double graph::RFunc::operator () (double x,double y,double z)
{
   trace("graph::RFunc::operator (x,y,z)");
   z /= 5;
   return z > -1 ? z > 1 ? 1 : (z + 1) / 2 : 0;
}

graph::Function3 *graph::RFunc::clone() const
{
   return new RFunc();
}

graph::Function3 *graph::GFunc::instance_ = NULL;

graph::Function3& graph::GFunc::getInstance()
{
   trace("graph::GFunc::getInstance");
   if ( ! instance_ ) {
      instance_ = new GFunc();
      instance_->ref();
   }
   return *instance_;
}

double graph::GFunc::operator () (double x,double y,double z)
{
   trace("graph::GFunc::operator (x,y,z)");
   z /= 5;
   return (z > -1 ? ( z < 0 ? z+1 : ( z < 1 ? 1-z : 0 ) ) : 0) / 2;
}

graph::Function3 *graph::GFunc::clone() const
{
   return new GFunc();
}

graph::Function3 *graph::BFunc::instance_ = NULL;

graph::Function3& graph::BFunc::getInstance()
{
   trace("graph::BFunc::getInstance");
   if ( ! instance_ ) {
      instance_ = new BFunc();
      instance_->ref();
   }
   return *instance_;
}

double graph::BFunc::operator () (double x,double y,double z)
{
   trace("graph::BFunc::operator (x,y,z)");
   z /= 5;
   return z > -1 ? z > 0 ? 0 : -z : 1;
}

graph::Function3 *graph::BFunc::clone() const
{
   return new BFunc();
}

//////////////////////////////////////////////////////////////////////////////////
//
// JGraph3d member definitions follow.
//


graph::JGraphable2::JGraphable2(JNIEnv *env,jobject jfunc):
   env_(env)
{
   jfunc_ = env->NewGlobalRef(jfunc);
   jclass clazz = env->GetObjectClass(jfunc_);
   mid_ = env_->GetMethodID(clazz,"call","(DD)D");
}

double graph::JGraphable2::operator () (double x,double y)
{
   return env_->CallDoubleMethod(jfunc_,mid_,x,y);
}

graph::Graph3d<100,100> *graph::JGraphable2::getGraph(double x0,
                                                      double x1,
                                                      double y0,
                                                      double y1,
                                                      int mode)
{
   return new JGraph3d(*this,x0,x1,y0,y1,mode);
}

graph::JGraphable2::~JGraphable2()
{
   env_->DeleteGlobalRef(jfunc_);
}

graph::JGraphable3::JGraphable3(JNIEnv *env,jobject jfunc):
   env_(env)
{
   jfunc_ = env->NewGlobalRef(jfunc);
   jclass clazz = env->GetObjectClass(jfunc_);
   mid_ = env_->GetMethodID(clazz,"call","(DDD)D");
}

double graph::JGraphable3::operator () (double x,double y,double z)
{
   return env_->CallDoubleMethod(jfunc_,mid_,x,y,z);
}

graph::Graph3d<100,100> *graph::JGraphable3::getGraph(double x0,
                                                      double x1,
                                                      double y0,
                                                      double y1,
                                                      int mode)
{
   return new JGraph3d(*this,x0,x1,y0,y1,mode);
}

graph::GraphableFunction3<100,100> *graph::JGraphable3::clone() const
{
   return new JGraphable3(env_,jfunc_);
}

graph::JGraphable3::~JGraphable3()
{
   env_->DeleteGlobalRef(jfunc_);
}

jclass graph::JGraphFactory::jfunc2_clazz_;
jclass graph::JGraphFactory::jfunc3_clazz_;
jclass graph::JGraphFactory::interpfunc_clazz_;
jfieldID graph::JGraphFactory::hfunction_id_;
bool graph::JGraphFactory::initialized_ = false;

//
// Beware: this method leaks memory.
//
graph::JGraph3d *graph::JGraphFactory::create(JNIEnv *env,
                                              jobject jfunc,
					      double x0,
					      double x1,
					      double y0,
					      double y1,
                                              int mode)
{
   if ( ! initialized_ ) { 
      jfunc2_clazz_ = env->FindClass("graph3d/Function2");
      jfunc3_clazz_ = env->FindClass("graph3d/Function3");
      interpfunc_clazz_ = env->FindClass("graph3d/peer/InterpretedFunction");
      hfunction_id_ = env->GetFieldID(interpfunc_clazz_,"handle_","J");
      if ( jfunc2_clazz_ && jfunc3_clazz_ && interpfunc_clazz_ && hfunction_id_ ) {
         jfunc2_clazz_ = (jclass) env->NewGlobalRef(jfunc2_clazz_);
	 jfunc3_clazz_ = (jclass) env->NewGlobalRef(jfunc3_clazz_);
         interpfunc_clazz_ = (jclass) env->NewGlobalRef(interpfunc_clazz_);
	 initialized_ = true;
      }
      else {
         throw initialization_error();
      }
   }
   if ( env->IsInstanceOf(jfunc,jfunc2_clazz_) ) {
      return new JGraph3d(* new JGraphable2(env,jfunc),x0,x1,y0,y1,mode);
   }
   if ( env->IsInstanceOf(jfunc,jfunc3_clazz_) ) {
      return new JGraph3d(* new JGraphable3(env,jfunc),x0,x1,y0,y1,mode);
   }
   if ( env->IsInstanceOf(jfunc,interpfunc_clazz_) ) {
         Function *function = (Function *) env->GetLongField(jfunc,hfunction_id_);
	 if ( ! strcmp(function->getType(),"(ff)f") ) {
            BCodeFunction2<100,100> *func =
	       new BCodeFunction2<100,100> (JGraph3d::getRuntime(),*function);
            return new JGraph3d(*func,x0,x1,y0,y1,mode);
	 }
	 if ( ! strcmp(function->getType(),"(fff)f") ) {
            BCodeFunction3<100,100> *func =
	       new BCodeFunction3<100,100> (JGraph3d::getRuntime(),*function);
            return new JGraph3d(*func,x0,x1,y0,y1,mode);
	 } 
   }
   throw typemismatch_error();
}

graph::Runtime graph::JGraph3d::rt_;

graph::JGraph3d::JGraph3d(GraphableFunction2<100,100>& f,
                          double x0,
                          double x1,
                          double y0,
                          double y1,
                          int mode):
   Graph3d<100,100>(f,x0,x1,y0,y1,mode),
   context_(NULL)
{
}

graph::JGraph3d::JGraph3d(GraphableFunction3<100,100>& f,
                          double x0,
                          double x1,
                          double y0,
                          double y1,
                          int mode):
   Graph3d<100,100>(f,x0,x1,y0,y1,mode),
   context_(NULL)
{
}

void graph::JGraph3d::initColors(float l0r,float l0g,float l0b,
                                 float l1r,float l1g,float l1b,
                                 float bgr,float bgg,float bgb)
{
   Graph3d<100,100>::initColors(l0r,l0g,l0b,l1r,l1g,l1b,bgr,bgg,bgb);
}

void graph::JGraph3d::realize(Display *dpy,
                              XVisualInfo *vis,
			      GLXDrawable d)
{
   context_ = glXCreateContext(dpy,vis,NULL,true);
   dpy_ = dpy;
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   init();
}

void graph::JGraph3d::display(GLXDrawable d)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::display();
   glXSwapBuffers(dpy_,d);
}

void graph::JGraph3d::reshape(GLXDrawable d,int width,int height)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::reshape(width,height);

}

void graph::JGraph3d::print(XVisualInfo *vis,
                            GLXDrawable d,
                            int width,
                            int height)
{
   GLXContext offscreen = glXCreateContext(dpy_,vis,NULL,false);
   assert( offscreen != NULL );
   if ( glXMakeCurrent(dpy_,d,offscreen) ) {
      Graph3d<100,100>::initSceneSettings();
      int dlists = Graph3d<100,100>::mode(DLISTS);
      Graph3d<100,100>::unsetMode(dlists);
      Graph3d<100,100>::reshape(width,height);
      Graph3d<100,100>::display();
      Graph3d<100,100>::setMode(dlists);
      glXWaitGL();
   }
   else {
      printf("offscreen context could not be made current!\n");
   }
   glXDestroyContext(dpy_,offscreen);
}

float graph::JGraph3d::getLightRedValue(int n) const
{
   return Graph3d<100,100>::getLightRedValue(n);
}

float graph::JGraph3d::getLightGreenValue(int n) const
{
   return Graph3d<100,100>::getLightGreenValue(n);
}

float graph::JGraph3d::getLightBlueValue(int n) const
{
   return Graph3d<100,100>::getLightBlueValue(n);
}

double graph::JGraph3d::getRangeX0() const
{
   return Graph3d<100,100>::getRangeX0();
}

double graph::JGraph3d::getRangeX1() const
{
   return Graph3d<100,100>::getRangeX1();
}

double graph::JGraph3d::getRangeY0() const
{
   return Graph3d<100,100>::getRangeY0();
}

double graph::JGraph3d::getRangeY1() const
{
   return Graph3d<100,100>::getRangeY1();
}

graph::Runtime& graph::JGraph3d::getRuntime()
{
   return rt_;
}

bool graph::JGraph3d::hasZ() const
{
   return Graph3d<100,100>::hasZ();
}

int graph::JGraph3d::mode(int m) const
{
   return Graph3d<100,100>::mode(m);
}

void graph::JGraph3d::screenRotate(GLXDrawable d,int x0,int y0,int x1,int y1)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::screenRotate(x0,y0,x1,y1);
}		      

void graph::JGraph3d::setRangeX(GLXDrawable d,double x0,double x1)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::setRangeX(x0,x1);
}

void graph::JGraph3d::setRangeY(GLXDrawable d,double y0,double y1)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::setRangeY(y0,y1);
}

void graph::JGraph3d::setZ(double z)
{
   Graph3d<100,100>::setZ(z);
}

void graph::JGraph3d::setClearColor(GLXDrawable d,float r,float g,float b)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::setClearColor(r,g,b);
}

void graph::JGraph3d::setLightColor(GLXDrawable d,int n,float r,float g,float b)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::setLightColor(n,r,g,b);
}

void graph::JGraph3d::setDefaultMode(GLXDrawable d)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::setDefaultMode();   
}

void graph::JGraph3d::setMode(GLXDrawable d,int mode)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::setMode(mode);  
   if ( mode & CONTOURS ) {
      Graph3d<100,100>::setRFunction(RFunc::getInstance());
      Graph3d<100,100>::setGFunction(GFunc::getInstance());
      Graph3d<100,100>::setBFunction(BFunc::getInstance());
      Graph3d<100,100>::initGraphValues();
   }
}

void graph::JGraph3d::rotateX(GLXDrawable d,double ax)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::rotateX(ax);
}

void graph::JGraph3d::rotateY(GLXDrawable d,double ay)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::rotateY(ay);
}

void graph::JGraph3d::rotateZ(GLXDrawable d,double az)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::rotateZ(az);
}

void graph::JGraph3d::unsetMode(GLXDrawable d,int mode)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::unsetMode(mode); 
   if ( mode & CONTOURS ) {
      Graph3d<100,100>::setRFunction(*DefaultRGBFunc::getInstance());
      Graph3d<100,100>::setGFunction(*DefaultRGBFunc::getInstance());
      Graph3d<100,100>::setBFunction(*DefaultRGBFunc::getInstance());
      Graph3d<100,100>::initGraphValues();
   }  
}

void graph::JGraph3d::zoom(GLXDrawable d,double tz)
{
   assert( context_ != NULL );
   glXMakeCurrent(dpy_,d,context_);
   Graph3d<100,100>::zoom(tz);
}

graph::JGraph3d::~JGraph3d()
{
   glXDestroyContext(dpy_,context_);
}



