// (C) by Thalion Graphic Solutions e.K. 2001
//
// Permissions to copy, modify and redistribute
// granted, according to the conditions defined
// in the Gnu Public License (GPL).
// Please read the file gpl.txt provided with this
// distribution.

#include <graph/runtime.h>
#include <math.h>
#include <stdarg.h>
#include <string.h>

#define MAGIC 0x2FEED

graph::Runtime::Ins::Ins(graph::Runtime::ins_t ins,unsigned par):
   ins_(ins),par_(par)
{
   // empty.
}

graph::Runtime::Ins::Ins(const Ins& ins)
{
   ins_ = ins.ins_;
   par_ = ins.par_;
}

graph::Runtime::Ins::Ins(Ins& ins)
{
   ins_ = ins.ins_;
   par_ = ins.par_;
}

graph::Runtime::Ins& graph::Runtime::Ins::operator=(const Ins& ins)
{
   ins_ = ins.ins_;
   par_ = ins.par_;
   return *this;
}

ostream& graph::operator << (ostream& os,graph::Runtime::Ins ins)
{
   char buf[30]; // Should be enough ...
   os << ins.print(buf);
   return os;
}

graph::Function::Function(const char *name,
                          const char *type,
			  const Runtime::Ins *cs,
                          size_t size,
			  int argc):
   magic_(MAGIC),name_(NULL),type_(NULL),cs_(NULL),argc_(argc)
{
   name_ = new char[strlen(name) + 1];
   strcpy(name_,name);
   type_ = new char[strlen(type) + 1];
   strcpy(type_,type);
   cs_ = new Runtime::Ins[size];
   memcpy(cs_,cs,sizeof(Runtime::Ins) * size);
}

#define PUSHARGS \
   bool skip = false; \
   for ( char *p = type_ + 1; *p; p++ ) { \
      if ( skip ) { \
         if ( *p == ')' ) { \
	    skip = false; \
	    env->push(va_arg(ap,Function *)); \
	    continue; \
	 } \
      } \
      switch ( *p ) { \
         case 'b': \
	 case 'i': \
	    env->push(va_arg(ap,int)); \
	    break; \
         case 'f': \
	    env->push(va_arg(ap,double)); \
	    break; \
	 case '(': \
	    skip = true; \
	    break; \
	 case ')': \
	    goto done; \
	 default: \
	    break; \
      } \
   } \
   done:
	 

double graph::Function::computeReal(graph::Runtime *env,...) const
{
   va_list ap;
   va_start(ap,env);
   env->reset();
   PUSHARGS
   va_end(ap);
   env->push(this);
   env->main(getCodeSegment());
   return env->getRealResult();
}

const graph::Function *graph::Function::computeFunction(graph::Runtime *env,...) const
{
   va_list ap;
   va_start(ap,env);
   env->reset();
   PUSHARGS
   va_end(ap);
   env->push(this);
   env->main(getCodeSegment());
   return env->getFunctionResult();
}

long graph::Function::computeInteger(graph::Runtime *env,...) const
{
   va_list ap;
   va_start(ap,env);
   env->reset();
   PUSHARGS
   va_end(ap);
   env->push(this);
   env->main(getCodeSegment());
   return env->getIntegerResult();
}

void graph::Function::print(FILE *stream) const
{
   char buf[30]; // Should be enough ...?
   fprintf(stream,"Function %s, type = %s:\n",getName(),getType());
   for ( Runtime::Ins *ins = cs_; ins->ins_ != Runtime::EOC; ins++ ) {
      fprintf(stream,"%s\n",ins->print(buf));
   }
}

graph::Function::~Function()
{
   delete[] name_;
   delete[] type_;
   delete[] cs_;
}

ostream& graph::operator << (ostream& os,const graph::Runtime::Ins *buf)
{
   for ( ; buf->ins_ != Runtime::EOC; buf++ ) {
      os << *buf << endl;
   }
   os << *buf << endl;
   return os;
}

ostream& graph::operator << (ostream& os,const graph::Function& function)
{
   os << "Function " << function.getName() << ", type = " << function.getType() << ':' << endl;
   os << function.getCodeSegment();
   return os;
}

graph::Runtime::Runtime(size_t ssize,size_t dsize):
   ss_(NULL),sp_(NULL),ds_(NULL),dsize_(dsize)
{
   ss_   = new Value[ssize];
   sp_   = ss_ - 1;
   stop_ = ss_ + ssize - 1;
   ds_   = new Value[dsize];
}

graph::Runtime::~Runtime()
{
   delete[] ss_;
   delete[] ds_;
}

#ifdef USE_ACC
# include "runtime_acc.cc"
#else
# include "runtime_stack.cc"
#endif
 
