// (C) by Thalion Graphic Solutions e.K. 2001
//
// Permissions to copy, modify and redistribute
// granted, according to the conditions defined
// in the Gnu Public License (GPL).
// Please read the file gpl.txt provided with this
// distribution.

/////////////////////////////////////////////////////////////////
// Only the parts of the engine depending on the instruction
// set and interpreter architecture are defined below.
//
// This is the plain stack machine variant.
/////////////////////////////////////////////////////////////////

#include <assert.h>

const char *graph::Runtime::Ins::print(char *buf) const
{
   switch ( ins_ ) {
      case Runtime::LDC:   sprintf(buf,"LDC\t%d",par_); return buf;
      case Runtime::LDI:   sprintf(buf,"LDI\t%d",par_); return buf;
      case Runtime::LDL:   sprintf(buf,"LDL\t%d",par_); return buf;
      case Runtime::LDC_PUSH:   sprintf(buf,"LDC_PUSH\t%d",par_); return buf;
      case Runtime::LDI_PUSH:   sprintf(buf,"LDI_PUSH\t%d",par_); return buf;
      case Runtime::LDL_PUSH:   sprintf(buf,"LDL_PUSH\t%d",par_); return buf;
      case Runtime::LDS:   sprintf(buf,"LDS\t"); return buf;
      case Runtime::STL:   sprintf(buf,"STL\t%d",par_); return buf;
      case Runtime::PUSH:  sprintf(buf,"PUSH"); return buf;
      case Runtime::PUSHC:  sprintf(buf,"PUSHC\t%d",par_); return buf;
      case Runtime::PUSHI:  sprintf(buf,"PUSHI\t%d",par_); return buf;
      case Runtime::PUSHL:  sprintf(buf,"PUSHL\t%d",par_); return buf;
      case Runtime::POP:   sprintf(buf,"POP"); return buf;
      case Runtime::DUP:   sprintf(buf,"DUP"); return buf;
      case Runtime::EXCH:  sprintf(buf,"EXCH"); return buf;
      case Runtime::CALL:  sprintf(buf,"CALL"); return buf;
      case Runtime::RET:   sprintf(buf,"RET"); return buf;
      case Runtime::J:     sprintf(buf,"J\t%d",par_); return buf;
      case Runtime::JT:    sprintf(buf,"JT\t%d",par_); return buf;
      case Runtime::JF:    sprintf(buf,"JF\t%d",par_); return buf;
      case Runtime::I2F:   sprintf(buf,"I2F"); return buf;
      case Runtime::F2I:   sprintf(buf,"F2I"); return buf;
      case Runtime::FADD:  sprintf(buf,"FADD"); return buf;
      case Runtime::FSUB:  sprintf(buf,"FSUB"); return buf;
      case Runtime::FMUL:  sprintf(buf,"FMUL"); return buf;
      case Runtime::FDIV:  sprintf(buf,"FDIV"); return buf;
      case Runtime::FPOW:  sprintf(buf,"FPOW"); return buf;
      case Runtime::FNEG:  sprintf(buf,"FNEG"); return buf;
      case Runtime::IADD:  sprintf(buf,"IADD"); return buf;
      case Runtime::ISUB:  sprintf(buf,"ISUB"); return buf;
      case Runtime::IMUL:  sprintf(buf,"IMUL"); return buf;
      case Runtime::IDIV:  sprintf(buf,"IDIV"); return buf;
      case Runtime::IMOD:  sprintf(buf,"IMOD"); return buf;
      case Runtime::IPOW:  sprintf(buf,"IPOW"); return buf;
      case Runtime::INEG:  sprintf(buf,"INEG"); return buf;
      case Runtime::EQ:    sprintf(buf,"EQ"); return buf;
      case Runtime::NE:    sprintf(buf,"NE"); return buf;
      case Runtime::IGT:   sprintf(buf,"IGT"); return buf;
      case Runtime::IGE:   sprintf(buf,"IGE"); return buf;
      case Runtime::ILT:   sprintf(buf,"ILT"); return buf;
      case Runtime::ILE:   sprintf(buf,"ILE"); return buf;
      case Runtime::FGT:   sprintf(buf,"FGT"); return buf;
      case Runtime::FGE:   sprintf(buf,"FGE"); return buf;
      case Runtime::FLT:   sprintf(buf,"FLT"); return buf;
      case Runtime::FLE:   sprintf(buf,"FLE"); return buf;
      case Runtime::AND:   sprintf(buf,"AND"); return buf;
      case Runtime::OR :   sprintf(buf,"OR"); return buf;
      case Runtime::XOR:   sprintf(buf,"XOR"); return buf;
      case Runtime::NOT:   sprintf(buf,"NOT"); return buf;
      case Runtime::SIN:   sprintf(buf,"SIN"); return buf;
      case Runtime::COS:   sprintf(buf,"COS"); return buf;
      case Runtime::TAN:   sprintf(buf,"TAN"); return buf;
      case Runtime::ASIN:  sprintf(buf,"ASIN"); return buf;
      case Runtime::ACOS:  sprintf(buf,"ACOS"); return buf;
      case Runtime::ATAN:  sprintf(buf,"ATAN"); return buf;
      case Runtime::EOC:   *buf = 0; return buf;
      default:    sprintf(buf,"<ILLEGAL INSTRUCTION>"); return buf;   
   }
}

#if USE_FAST_DISPATCH
# if __GNUG__
#  define SWITCH const void *labels[] = { \
 &&_LDC, &&_LDI, &&_LDL, &&_LDC_PUSH, &&_LDI_PUSH, &&_LDL_PUSH, &&_LDS, &&_STL, \
 &&_PUSH, &&_PUSHC, &&_PUSHI, &&_PUSHL, &&_POP, &&_DUP, &&_EXCH, &&_CALL, &&_RET, \
 &&_J, &&_JT, &&_JF, &&_F2I, &&_I2F, &&_FADD, &&_FSUB, &&_FMUL, &&_FDIV, &&_FPOW, \
 &&_FNEG, &&_IADD, &&_ISUB, &&_IMUL, &&_IDIV, &&_IMOD, &&_IPOW, &&_INEG, &&_EQ, \
 &&_NE, &&_IGT, &&_IGE, &&_ILT, &&_ILE, &&_FGT, &&_FGE, &&_FLT, &&_FLE, &&_AND, \
 &&_OR, &&_XOR, &&_NOT, &&_SIN, &&_COS, &&_TAN, &&_ASIN, &&_ACOS, &&_ATAN, \
 &&_EOC, &&_EOC, &&_EOC, &&_EOC, &&_EOC, &&_EOC, &&_EOC, &&_EOC, &&_EOC \
 }; \
 goto *labels[ip->ins_];
#  define CASE(label)   _##label
#  define DEFAULT       _DUMMY
#  define BREAK         goto *labels[ip->ins_]
# else
#  error "USE_FAST_DISPATCH only supported on gnu C++"
# endif
#else
# define SWITCH switch ( ip->ins_ )
# define CASE(label)   case label
# define DEFAULT       default
# define BREAK         break
#endif

void graph::Runtime::main(const graph::Runtime::Ins *cs)
{
   Value        acc;           // Accumulator.
   const Ins   *ip = cs;       // Instruction pointer.
   Value       *frame = sp_-1, // Frame pointer (local ref = frame - ip->par_).
               *sp = sp_;      // Stack pointer.
   (++sp)->v_ = NULL;          // Push NULL frame.
   (++sp)->i_ = 0;             // Push any IP (will be dropped anyway; just for
                               // stack consistence.
   for ( ;; ) {
      if ( sp > stop_ ) {
         throw overflow();
      }
      if ( sp < sp_ ) {
         throw underflow();
      }
      SWITCH {
         CASE( LDC ):
            acc = ds_[ip->par_];
            ip ++;
            BREAK;
	 CASE( LDI ):
            acc.l_ = ip->par_;
            ip ++;
            BREAK;
         CASE( LDL ):
            acc = *(frame - ip->par_);
            ip ++;
            BREAK;
         CASE( LDC_PUSH ):
            *(++sp) = acc;
            acc = ds_[ip->par_];
            ip ++;
            BREAK;
	 CASE( LDI_PUSH ):
            *(++sp) = acc;
            acc.l_ = ip->par_;
            ip ++;
            BREAK;
         CASE( LDL_PUSH ):
            *(++sp) = acc;
            acc = *(frame - ip->par_);
            ip ++;
            BREAK;
         CASE( LDS ):
            acc = *sp;
            ip ++;
            BREAK;
         CASE( STL ):
            *(frame - ip->par_) = acc;
            ip ++;
            BREAK;
         CASE( PUSH ):
            *(++sp) = acc;
            ip ++;
            BREAK;
         CASE( PUSHC ):
            *(++sp) = ds_[ip->par_];
            ip ++;
            BREAK;
	 CASE( PUSHI ):
            (++sp)->l_ = ip->par_;
            ip ++;
            BREAK;
         CASE( PUSHL ):
            *(++sp) = *(frame - ip->par_);
            ip ++;
            BREAK;
         CASE( POP ):
            sp--;
            ip++;
            BREAK;
         CASE( DUP ):
            *sp = *(sp - 1);
            ip++;
            BREAK; 
         CASE( EXCH ): {
            Value v = *sp;
            *sp = *(sp - 1);
            *(sp - 1) = v;
            ip++;
            BREAK;
         }
         CASE( CALL ): { // Stack: <arg1> ... <argN>
                         // acc  : <function>
            (++sp)->f_ = acc.f_;
            (++sp)->v_ = frame;
            (++sp)->i_ = ip + 1;
            frame = sp - 3;
            ip = acc.f_->getCodeSegment();
            BREAK;   // Stack: <arg1> ... <argN> <function> <frame> <ip>
                     // acc  : <function>
         } 
         CASE( RET ): {  // Stack: <arg1> ... <argN> <function> <frame> <ip>
                         // acc  : <result>
            ip = sp-- ->i_;
            frame = sp-- ->v_;
            const Function *f = sp-- ->f_;
            sp -= f->getArgC();
	    if ( ! frame ) { // We returned from toplevel call.
               *(++sp) = acc;
               return;
            }
            BREAK;   // Stack:
                     // acc  : <result>
	 }
         CASE( J ):
            ip = (Ins *) (cs + ip->par_);
            BREAK;
         CASE( JT ):
            ip = acc.l_ ? (Ins *) (cs + ip->par_) : ip + 1;  
            BREAK;
         CASE( JF ):
            ip = acc.l_ ? ip + 1 : (Ins *) (cs + ip->par_);  
            BREAK;
         CASE( F2I ):
            acc.l_ = (long) acc.d_;
            ip++;
            BREAK;
         CASE( I2F ):
            acc.d_ = (double) acc.l_;
            ip++;
            BREAK;
         CASE( FADD ):
            acc.d_ += sp->d_;
            sp--;
            ip++;
            BREAK;
         CASE( FSUB ):
            acc.d_ = -acc.d_;
            acc.d_ += sp->d_;
            sp--;
            ip++;
            BREAK;
         CASE( FMUL ):
            acc.d_ *= sp->d_;
            sp--;
            ip++;
            BREAK;
         CASE( FDIV ): {
            if ( acc.l_ ) {
               acc.d_ = sp->d_ / acc.d_;
               sp--;
	       ip++;
            }
            else {
               sp--;
               throw division_by_zero();
            }
            BREAK;
         }
         CASE( FPOW ):
            acc.d_ = pow(sp->d_,acc.d_);
            sp--;
            ip++;
            BREAK;
         CASE( FNEG ):
            acc.d_ = -acc.d_;
            ip++;
            BREAK;
         CASE( IADD ):
            acc.l_ += sp->l_;
            sp--;
            ip++;
            BREAK;
         CASE( ISUB ):
            acc.l_ = -acc.l_;
            acc.l_ += sp->l_;
            sp--;
            ip++;
            BREAK;
         CASE( IMUL ):
            acc.l_ *= sp->l_;
            sp--;
            ip++;
            BREAK;
         CASE( IDIV ): {
            if ( acc.l_  ) {
               acc.l_ = sp->l_ / acc.l_;
               sp--;
	       ip++;
            }
            else {
               sp--;
               throw division_by_zero();
            }
            BREAK;
         }
         CASE( IMOD ): {
            if ( acc.l_ ) {
               acc.l_ = sp->l_ % acc.l_;
               sp--;
	       ip++;
            }
            else {
               sp--;
               throw division_by_zero();
            }
            BREAK;
         }
         CASE( IPOW ):
            acc.l_ = pow(sp->l_,acc.l_);
            sp--;
            ip++;
            BREAK;
         CASE( INEG ):
            acc.l_ = -acc.l_;
            ip++;
            BREAK;
         CASE( EQ ):
            acc.l_ = acc.l_ == sp->l_;
            sp--;
            ip++;
            BREAK;
         CASE( NE ):
            acc.l_ != acc.l_ == sp->l_;
            sp--;
            ip++;
            BREAK;
         CASE( IGT ):
            acc.l_ = sp->l_ > acc.l_;
            sp--;
            ip++;
            BREAK;
         CASE( IGE ):
            acc.l_ = sp->l_ >= acc.l_;
            sp--;
            ip++;
            BREAK;
         CASE( ILT ):
            acc.l_ = sp->l_ < acc.l_;
            sp--;
            ip++;
            BREAK;
         CASE( ILE ):
            acc.l_ = sp->l_ <= acc.l_;
            sp--;
            ip++;
            BREAK;
         CASE( FGT ):
            acc.l_ = sp->d_ > acc.d_;
            sp--;
            ip++;
            BREAK;
         CASE( FGE ):
            acc.d_ = sp->d_ >= acc.d_;
            sp--;
            ip++;
            BREAK;
         CASE( FLT ):
            acc.d_ = sp->d_ < acc.d_;
            sp--;
            ip++;
            BREAK;
         CASE( FLE ):
            acc.d_ = sp->d_ <= acc.d_;
            sp--;
            ip++;
            BREAK;
         CASE( AND ):
            acc.l_ = acc.l_ && sp->l_;
            sp--;
            ip++;
            BREAK;
         CASE( OR ):
            acc.l_ = acc.l_ || sp->l_;
            sp--;
            ip++;
            BREAK;
         CASE( XOR ):
            acc.l_ = acc.l_ ^ sp->l_;
            sp--;
            ip++;
            BREAK;
         CASE( NOT ):
            acc.l_ = ! acc.l_;
            ip++;
            BREAK;
         CASE( SIN ):
            acc.d_ = sin(acc.d_);
            ip++;
            BREAK;
         CASE( COS ):
            acc.d_ = cos(acc.d_);
            ip++;
            BREAK;
         CASE( TAN ):
            acc.d_ = tan(acc.d_);
            ip++;
            BREAK;
         CASE( ASIN ):
            acc.d_ = asin(acc.d_);
            ip++;
            BREAK;
         CASE( ACOS ):
            acc.d_ = acos(acc.d_);
            ip++;
            BREAK;
         CASE( ATAN ):
            acc.d_ = atan(acc.d_);
            ip++;
            BREAK;
         CASE ( EOC ):
            return;
         DEFAULT:
            throw illegal_instruction();
      }
   }
}

