// (C) by Thalion Graphic Solutions e.K. 2001
//
// Permissions to copy, modify and redistribute
// granted, according to the conditions defined
// in the Gnu Public License (GPL).
// Please read the file gpl.txt provided with this
// distribution.

/////////////////////////////////////////////////////////////////
// Only the parts of the engine depending on the instruction
// set and interpreter architecture are defined below.
//
// This is the plain stack machine variant.
/////////////////////////////////////////////////////////////////

const char *graph::Runtime::Ins::print(char *buf) const
{
   switch ( ins_ ) {
      case Runtime::PUSHC: sprintf(buf,"PUSHC\t%d",par_); return buf;
      case Runtime::PUSHI: sprintf(buf,"PUSHI\t%d",par_); return buf;
      case Runtime::PUSHL: sprintf(buf,"PUSHL\t%d",par_); return buf;
      case Runtime::STL:   sprintf(buf,"STL\t%d",par_); return buf;
      case Runtime::POP:   sprintf(buf,"POP"); return buf;
      case Runtime::DUP:   sprintf(buf,"DUP"); return buf;
      case Runtime::EXCH:  sprintf(buf,"EXCH"); return buf;
      case Runtime::CALL:  sprintf(buf,"CALL"); return buf;
      case Runtime::RET:   sprintf(buf,"RET"); return buf;
      case Runtime::J:     sprintf(buf,"J\t%d",par_); return buf;
      case Runtime::JT:    sprintf(buf,"JT\t%d",par_); return buf;
      case Runtime::JF:    sprintf(buf,"JF\t%d",par_); return buf;
      case Runtime::I2F:   sprintf(buf,"I2F"); return buf;
      case Runtime::F2I:   sprintf(buf,"F2I"); return buf;
      case Runtime::FADD:  sprintf(buf,"FADD"); return buf;
      case Runtime::FSUB:  sprintf(buf,"FSUB"); return buf;
      case Runtime::FMUL:  sprintf(buf,"FMUL"); return buf;
      case Runtime::FDIV:  sprintf(buf,"FDIV"); return buf;
      case Runtime::FPOW:  sprintf(buf,"FPOW"); return buf;
      case Runtime::FNEG:  sprintf(buf,"FNEG"); return buf;
      case Runtime::IADD:  sprintf(buf,"IADD"); return buf;
      case Runtime::ISUB:  sprintf(buf,"ISUB"); return buf;
      case Runtime::IMUL:  sprintf(buf,"IMUL"); return buf;
      case Runtime::IDIV:  sprintf(buf,"IDIV"); return buf;
      case Runtime::IMOD:  sprintf(buf,"IMOD"); return buf;
      case Runtime::IPOW:  sprintf(buf,"IPOW"); return buf;
      case Runtime::INEG:  sprintf(buf,"INEG"); return buf;
      case Runtime::EQ:    sprintf(buf,"EQ"); return buf;
      case Runtime::NE:    sprintf(buf,"NE"); return buf;
      case Runtime::IGT:   sprintf(buf,"IGT"); return buf;
      case Runtime::IGE:   sprintf(buf,"IGE"); return buf;
      case Runtime::ILT:   sprintf(buf,"ILT"); return buf;
      case Runtime::ILE:   sprintf(buf,"ILE"); return buf;
      case Runtime::FGT:   sprintf(buf,"FGT"); return buf;
      case Runtime::FGE:   sprintf(buf,"FGE"); return buf;
      case Runtime::FLT:   sprintf(buf,"FLT"); return buf;
      case Runtime::FLE:   sprintf(buf,"FLE"); return buf;
      case Runtime::AND:   sprintf(buf,"AND"); return buf;
      case Runtime::OR :   sprintf(buf,"OR"); return buf;
      case Runtime::XOR:   sprintf(buf,"XOR"); return buf;
      case Runtime::NOT:   sprintf(buf,"NOT"); return buf;
      case Runtime::SIN:   sprintf(buf,"SIN"); return buf;
      case Runtime::COS:   sprintf(buf,"COS"); return buf;
      case Runtime::TAN:   sprintf(buf,"TAN"); return buf;
      case Runtime::ASIN:  sprintf(buf,"ASIN"); return buf;
      case Runtime::ACOS:  sprintf(buf,"ACOS"); return buf;
      case Runtime::ATAN:  sprintf(buf,"ATAN"); return buf;
      case Runtime::EOC:   *buf = 0; return buf;
      default:    sprintf(buf,"<ILLEGAL INSTRUCTION>"); return buf;   
   }
}

void graph::Runtime::main(const graph::Runtime::Ins *cs)
{
   const Ins   *ip = cs;       // Instruction pointer.
   Value       *frame = sp_-1, // Frame pointer (local ref = frame - ip->par_).
               *sp = sp_;      // Stack pointer.
   (++sp)->v_ = NULL;          // Push NULL frame.
   (++sp)->i_ = 0;             // Push any IP (will be dropped anyway; just for
                               // stack consistence.
   (++sp)->i_ = 0;             // The same is due to the cs. 			     
   for ( ;; ) {
      // char buf[30]; printf("%x(%x): %s\n", cs, ip, ip->print(buf));
      if ( sp > stop_ ) {
         throw overflow();
      }
      if ( sp < sp_ ) {
         throw underflow();
      }
      switch ( ip->ins_ ) {
         case PUSHC:
            *(++sp) = ds_[ip->par_];
            ip ++;
            break;
	 case PUSHI:
            (++sp)->l_ = ip->par_;
            ip ++;
            break;
         case PUSHL:
            *(++sp) = *(frame - ip->par_);
            ip ++;
            break;
         case STL:
            *(frame - ip->par_) = *sp--;
            ip ++;
            break;
         case POP:
            sp--;
            ip++;
            break;
         case DUP:
            *sp = *(sp - 1);
            ip++;
            break; 
         case EXCH: {
            Value v = *sp;
            *sp = *(sp - 1);
            *(sp - 1) = v;
            ip++;
            break;
         }
         case CALL: { // Stack: <arg1> ... <argN> <function>
            const Function *f = sp->f_;
            (++sp)->v_ = frame;
            (++sp)->i_ = ip + 1;
	    (++sp)->i_ = cs;
            frame = sp - 4;
            cs = ip = f->getCodeSegment();
            break;   // Stack: <arg1> ... <argN> <function> <frame> <ip> <cs>
         } 
         case RET: {  // Stack: <arg1> ... <argN> <function> <frame> <ip> <cs> <result>
            Value ret = *sp--;
	    cs = sp-- ->i_;
            ip = sp-- ->i_;
            frame = sp-- ->v_;
            const Function *f = sp-- ->f_;
            sp -= f->getArgC();
            *(++sp) = ret;
	    if ( ! frame ) { // We returned from toplevel call.
               return;
            }
            break;   // Stack: <result>
	 }
         case J:
            ip = (Ins *) (cs + ip->par_);
            break;
         case JT:
            ip = sp-- ->l_ ? (Ins *) (cs + ip->par_) : ip + 1;  
            break;
         case JF:
            ip = sp-- ->l_ ? ip + 1 : (Ins *) (cs + ip->par_);  
            break;
         case F2I:
            sp->l_ = (long) sp->d_;
            ip++;
            break;
         case I2F:
            sp->d_ = (double) sp->l_;
            ip++;
            break;
         case FADD:
            (--sp)->d_ = sp->d_ + sp[1].d_;
            ip++;
            break;
         case FSUB:
            (--sp)->d_ = sp->d_ - sp[1].d_;
            ip++;
            break;
         case FMUL:
            (--sp)->d_ = sp->d_ * sp[1].d_;
            ip++;
            break;
         case FDIV: {
            if ( sp->d_ != 0 ) {
               (--sp)->d_ = sp->d_ / sp[1].d_;
	       ip++;
            }
            else {
               (--sp)->l_ = 0;
               throw division_by_zero();
            }
            break;
         }
         case FPOW:
            (--sp)->d_ = pow(sp->d_,sp[1].d_);
            ip++;
            break;
         case FNEG:
            sp->d_ = -sp->d_;
            ip++;
            break;
         case IADD:
            (--sp)->l_ = sp->l_ + sp[1].l_;
            ip++;
            break;
         case ISUB:
            (--sp)->l_ = sp->l_ - sp[1].l_;
            ip++;
            break;
         case IMUL:
            (--sp)->l_ = sp->l_ * sp[1].l_;
            ip++;
            break;
         case IDIV: {
            if ( sp->l_ != 0 ) {
               (--sp)->l_ = sp->l_ / sp[1].l_;
	       ip++;
            }
            else {
               (--sp)->l_ = 0;
               throw division_by_zero();
            }
            break;
         }
         case IMOD: {
            if ( sp->l_ != 0 ) {
               (--sp)->l_ = sp->l_ % sp[1].l_;
	       ip++;
            }
            else {
               (--sp)->l_ = 0;
               throw division_by_zero();
            }
            break;
         }
         case IPOW:
            (--sp)->l_ = pow(sp->l_,sp[1].l_);
            ip++;
            break;
         case INEG:
            sp->l_ = -sp->l_;
            ip++;
            break;
         case EQ:
            (--sp)->l_ = sp->l_ == sp[1].l_;
            ip++;
            break;
         case NE:
            (--sp)->l_ != sp->l_ == sp[1].l_;
            ip++;
            break;
         case IGT:
            (--sp)->l_ = sp->l_ > sp[1].l_;
            ip++;
            break;
         case IGE:
            (--sp)->l_ = sp->l_ >= sp[1].l_;
            ip++;
            break;
         case ILT:
            (--sp)->l_ = sp->l_ < sp[1].l_;
            ip++;
            break;
         case ILE:
            (--sp)->l_ = sp->l_ <= sp[1].l_;
            ip++;
            break;
         case FGT:
            (--sp)->d_ = sp->d_ > sp[1].d_;
            ip++;
            break;
         case FGE:
            (--sp)->d_ = sp->d_ >= sp[1].d_;
            ip++;
            break;
         case FLT:
            (--sp)->d_ = sp->d_ < sp[1].d_;
            ip++;
            break;
         case FLE:
            (--sp)->d_ = sp->d_ <= sp[1].d_;
            ip++;
            break;
         case AND:
            (--sp)->l_ = sp->l_ && sp[1].l_;
            ip++;
            break;
         case OR:
            (--sp)->l_ = sp->l_ || sp[1].l_;
            ip++;
            break;
         case XOR:
            (--sp)->l_ = sp->l_ ^ sp[1].l_;
            ip++;
            break;
         case NOT:
            sp->l_ = ! sp->l_;
            ip++;
            break;
         case SIN:
            sp->d_ = sin(sp->d_);
            ip++;
            break;
         case COS:
            sp->d_ = cos(sp->d_);
            ip++;
            break;
         case TAN:
            sp->d_ = tan(sp->d_);
            ip++;
            break;
         case ASIN:
            sp->d_ = asin(sp->d_);
            ip++;
            break;
         case ACOS:
            sp->d_ = acos(sp->d_);
            ip++;
            break;
         case ATAN:
            sp->d_ = atan(sp->d_);
            ip++;
            break;
         default:
            throw illegal_instruction();
      }
   }
}

