#include <graph/codegen.h>
#include <graph/function.h>
#include <graph/graph3d.h>

#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>

#include "trace.h"

typedef graph::Graph3d<> g3d;

const double PI = 3.14159;

class TestFunction: public graph::BCodeFunction3<100,100> {
   public:
      static TestFunction *create(graph::Runtime&);
   protected:
      TestFunction(graph::Runtime&,const graph::Function&);
};

TestFunction *TestFunction::create(graph::Runtime& env)
{
   graph::CodeGen cg = env;
   unsigned fp = cg.beginFunction("f","(fff)f",3);
   /*  0 */  cg.addIns(graph::Runtime::LDI,5);   // acc:5
   /*  1 */  cg.addIns(graph::Runtime::I2F);     // acc:5.0
   /*  2 */  cg.addIns(graph::Runtime::PUSH);    // acc:5.0; stack: 5.0
   /*  3 */  cg.addIns(graph::Runtime::PUSHL,0);   // acc: x  ; stack: 5.0
   /*  4 */  cg.addIns(graph::Runtime::LDI,2);   // acc: 2  ; stack: 5.0,x
   /*  5 */  cg.addIns(graph::Runtime::I2F);     // acc: 2.0; stack: 5.0,x
   /*  6 */  cg.addIns(graph::Runtime::FDIV);    // acc: x/2.0;stack: 5.0
   /*  7 */  cg.addIns(graph::Runtime::COS);     // acc: cos(x/2.0);stack: 5.0
   /*  8 */  cg.addIns(graph::Runtime::PUSH);    // acc: cos(x/2.0); stack: 5.0,cos(x/2.0)
   /*  9 */  cg.addIns(graph::Runtime::PUSHL,1);   // acc: y  ; stack: 5.0,cos(x/2.0)
   /* 10 */  cg.addIns(graph::Runtime::LDI,2);   // acc: 2  ; stack: 5.0,cos(x/2.0),y
   /* 11 */  cg.addIns(graph::Runtime::I2F);     // acc: 2.0; stack: 5.0,cos(x/2.0),y
   /* 12 */  cg.addIns(graph::Runtime::FDIV);    // acc: y/2.0; stack: 5.0,cos(x/2.0)
   /* 13 */  cg.addIns(graph::Runtime::COS);     // acc: cos(y/2.0); stack: 5.0,cos(x/2.0)
   /* 14 */  cg.addIns(graph::Runtime::LDL_PUSH,2);   // acc: z  ; stack: 5.0,cos(x/2.0),cos(y/2.0) 
   /* 15 */  cg.addIns(graph::Runtime::COS);     // acc: cos(z); stack: 5.0,cos(x/2.0),cos(y/2.0) 
   /* 16 */  cg.addIns(graph::Runtime::FMUL);    // acc: cos(y/2.0)*cos(z); stack: 5.0,cos(x/2.0)
   /* 17 */  cg.addIns(graph::Runtime::FMUL);    // acc: cos(x/2.0)*cos(y/2.0)*cos(z); stack: 5.0
   /* 18 */  cg.addIns(graph::Runtime::FMUL);    // acc: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z); stack:
   /* 19 */  cg.addIns(graph::Runtime::LDI_PUSH,7);   // acc: 7; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)
   /* 20 */  cg.addIns(graph::Runtime::I2F);     // acc: 7.0; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)
   /* 21 */  cg.addIns(graph::Runtime::PUSH);    // acc: 7.0; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z),7.0
   /* 22 */  cg.addIns(graph::Runtime::PUSHL,0);   // acc: x; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z),7.0
   /* 23 */  cg.addIns(graph::Runtime::LDL,2);   // acc: z; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z),7.0,x
   /* 24 */  cg.addIns(graph::Runtime::FADD);    // acc: x+z; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z),7.0
   /* 25 */  cg.addIns(graph::Runtime::FMUL);    // acc: 7.0*(x+z); stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)
   /* 26 */  cg.addIns(graph::Runtime::SIN);     // acc: sin(7.0*(x+z)); stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)
   /* 27 */  cg.addIns(graph::Runtime::LDI_PUSH,6);   // acc: 6
   /* 28 */  cg.addIns(graph::Runtime::I2F);     // acc: 6.0
   /* 29 */  cg.addIns(graph::Runtime::FDIV);    // acc: sin(7.0*(x+z))/6.0; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)
   /* 30 */  cg.addIns(graph::Runtime::LDI_PUSH,9); // 9
   /* 31 */  cg.addIns(graph::Runtime::I2F);     // 9 - 9.0
   /* 32 */  cg.addIns(graph::Runtime::PUSH);
   /* 33 */  cg.addIns(graph::Runtime::PUSHL,0); // x
   /* 34 */  cg.addIns(graph::Runtime::LDI,2); // 2
   /* 35 */  cg.addIns(graph::Runtime::I2F);     // 2 -> 2.0
   /* 36 */  cg.addIns(graph::Runtime::FDIV);    // /
   /* 37 */  cg.addIns(graph::Runtime::PUSH);
   /* 38 */  cg.addIns(graph::Runtime::PUSHL,2); // z
   /* 39 */  cg.addIns(graph::Runtime::LDI,3); // 3
   /* 40 */  cg.addIns(graph::Runtime::I2F);     // 3 - 3.0
   /* 41 */  cg.addIns(graph::Runtime::FDIV);    // /
   /* 42 */  cg.addIns(graph::Runtime::LDL_PUSH,1); // y
   /* 43 */  cg.addIns(graph::Runtime::FADD);    // +
   /* 44 */  cg.addIns(graph::Runtime::FADD);    // +
   /* 45 */  cg.addIns(graph::Runtime::FMUL);    // *
   /* 46 */  cg.addIns(graph::Runtime::SIN);     // sin
   /* 47 */  cg.addIns(graph::Runtime::LDI_PUSH,7); // 7
   /* 48 */  cg.addIns(graph::Runtime::I2F);     // 7 -> 7.0
   /* 49 */  cg.addIns(graph::Runtime::FDIV);    // /
   /* 50 */  cg.addIns(graph::Runtime::FADD);    // +
   /* 51 */  cg.addIns(graph::Runtime::FADD);    // +
   /* 52 */  cg.addIns(graph::Runtime::RET);     // return
   cg.endFunction();
   const graph::Function *f = cg.getFunction();
   cout << *f << endl;
   return new TestFunction(env,*f);
}

TestFunction::TestFunction(graph::Runtime& env,const graph::Function& f):
   graph::BCodeFunction3<100,100>(env,f)
{
}

class RFunc: public graph::Function3 {
   public:
      double operator () (double,double,double);
      static graph::Function3 *getInstance();
   private:
      static graph::Function3 *instance_;
};

graph::Function3 *RFunc::instance_ = NULL;

double RFunc::operator () (double x,double y,double z)
{
   z /= 5;
   return z > -1 ? z > 1 ? 1 : (z + 1) / 2 : 0;
}

graph::Function3 *RFunc::getInstance()
{
   if ( ! instance_ ) {
      instance_ = new RFunc();
      instance_->ref();
   }
   return instance_;
}


class GFunc: public graph::Function3 {
   public:
      double operator () (double,double,double);
      static graph::Function3 *getInstance();
   private:
      static graph::Function3 *instance_;
};

graph::Function3 *GFunc::instance_ = NULL;

double GFunc::operator () (double x,double y,double z)
{
   z /= 5;
   return (z > -1 ? ( z < 0 ? z+1 : ( z < 1 ? 1-z : 0 ) ) : 0) / 2;
}

graph::Function3 *GFunc::getInstance()
{
   if ( ! instance_ ) {
      instance_ = new GFunc();
      instance_->ref();
   }
   return instance_;
}

class BFunc: public graph::Function3 {
   public:
      double operator () (double,double,double);
      static graph::Function3 *getInstance();
   private:
      static graph::Function3 *instance_;
};

graph::Function3 *BFunc::instance_ = NULL;

double BFunc::operator () (double x,double y,double z)
{
   z /= 5;
   return z > -1 ? z > 0 ? 0 : -z : 1;
}

graph::Function3 *BFunc::getInstance()
{
   if ( ! instance_ ) {
      instance_ = new BFunc();
      instance_->ref();
   }
   return instance_;
}

class Graph3dTest {
   public:
      static void init();
      static void display();
      static void reshape(int,int);
      static void animate(int);
   private:
      static graph::Graph3d<100,100> *graph_;
      static graph::Runtime           env_;
};

void Graph3dTest::init()
{
   TestFunction *f = TestFunction::create(env_);
   graph_ = new graph::Graph3d<100,100>(*f,-PI,PI,-PI,PI);
   graph_->init();
   graph_->setMode(graph::Graph3d<100,100>::SMOOTH |
                   graph::Graph3d<100,100>::AXES |
		   graph::Graph3d<100,100>::DLISTS);
}

void Graph3dTest::display()
{
   graph_->display();
   glutSwapBuffers();
}

void Graph3dTest::reshape(int w,int h)
{
   graph_->reshape(w,h);
}

void Graph3dTest::animate(int /* unused */)
{
   static int mode = 0;
   static int step = 0;
   switch ( mode ) {
      case 0:
         cout << "Setting mode to flat." << endl;
         graph_->setRFunction(*graph::DefaultRGBFunc::getInstance());
         graph_->setGFunction(*graph::DefaultRGBFunc::getInstance());
         graph_->setBFunction(*graph::DefaultRGBFunc::getInstance());      
         break;   
      case 1:
         cout << "Adding contour shades." << endl;
         graph_->setRFunction(*RFunc::getInstance());
         graph_->setGFunction(*GFunc::getInstance());
         graph_->setBFunction(*BFunc::getInstance());      
         break;   
      case 2:
         cout << "Switching lights on." << endl;
         graph_->setMode(graph::Graph3d<100,100>::LIGHTS);
	 break;
      default:
         mode = 0;
         graph_->unsetMode(graph::Graph3d<100,100>::LIGHTS);
	 break;
   }
   step++;
   if ( step % 50 == 0 ) {
      mode++;
   } 
   graph_->setZ(PI/100*step);
   graph_->rotateX(1.0);
   graph_->rotateY(1.0);
   graph_->rotateZ(1.0);
   glutPostRedisplay();
   glutTimerFunc(50,&Graph3dTest::animate, NULL);
}

graph::Graph3d<100,100> *Graph3dTest::graph_ = NULL;
graph::Runtime           Graph3dTest::env_;

int main(int argc,char *argv[])
{
   glutInit(&argc,argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(500,500);
   glutCreateWindow(argv[0]);
   Graph3dTest::init();
   glutDisplayFunc(&Graph3dTest::display);
   glutReshapeFunc(&Graph3dTest::reshape);
   glutTimerFunc(50,&Graph3dTest::animate, NULL);
   glutMainLoop();
   return 0;
}

