#include <graph/codegen.h>

using graph::CodeGen;
using graph::Function;
using graph::Runtime;

int main()
{
   Runtime env;
   CodeGen cg(env);
   
   // Fibonacci function.
   unsigned fp = cg.beginFunction("fib","(i)i");
#ifdef USE_ACC
   // if
   //    n < 2
           

   /*  0 */  cg.addIns(Runtime::PUSHL,0); // stack=n           ;acc=
   /*  1 */  cg.addIns(Runtime::LDI,2);   // stack=n           ;acc=2
   /*  2 */  cg.addIns(Runtime::ILT);     // stack=            ;acc=n<2
   /*  3 */  cg.addIns(Runtime::JF,6);    // stack=            ;acc=n<2
   // then
   //    1
   /*  4 */  cg.addIns(Runtime::LDI,1);
   /*  5 */  cg.addIns(Runtime::RET);
   // else
   //    fib(n-1) + fib(n-2)
   /*  6 */  cg.addIns(Runtime::PUSHL,0); // stack=n           ;acc=
   /*  7 */  cg.addIns(Runtime::LDI,1);   // stack=n           ;acc=1
   /*  8 */  cg.addIns(Runtime::ISUB);    // stack=            ;acc=n-1
   /*  9 */  cg.addIns(Runtime::LDC_PUSH,fp);  // stack=n-1         ;acc=fib
   /* 10 */  cg.addIns(Runtime::CALL);    // stack=            ;acc=fib(n-1)
   /* 11 */  cg.addIns(Runtime::LDI_PUSH,2);   // stack=fib(n-1),n  ;acc=2
   /* 12 */  cg.addIns(Runtime::PUSHL,0); // stack=fib(n-1),n  ;acc=fib(n-1)
   /* 13 */  cg.addIns(Runtime::ISUB);    // stack=fib(n-1)    ;acc=n-2
   /* 14 */  cg.addIns(Runtime::LDC_PUSH,fp);  // stack=fib(n-1),n-2;acc=fib
   /* 15 */  cg.addIns(Runtime::CALL);    // stack=fib(n-1)    ;acc=fib(n-2)
   /* 16 */  cg.addIns(Runtime::IADD);    // stack=            ;acc=fib(n-1)+fib(n-2)
   /* 17 */  cg.addIns(Runtime::RET);
#else
   // if
   //    n < 2
   /*  0 */  cg.addIns(Runtime::PUSHL,0);
   /*  1 */  cg.addIns(Runtime::PUSHI,2);
   /*  2 */  cg.addIns(Runtime::ILT);
   /*  3 */  cg.addIns(Runtime::JF,6);
   // then
   //    1
   /*  4 */  cg.addIns(Runtime::PUSHI,1);
   /*  5 */  cg.addIns(Runtime::RET);
   // else
   //    fib(n-1) + fib(n-2)
   /*  6 */  cg.addIns(Runtime::PUSHL,0);
   /*  7 */  cg.addIns(Runtime::PUSHI,1);
   /*  8 */  cg.addIns(Runtime::ISUB);
   /*  9 */  cg.addIns(Runtime::PUSHC,fp);
   /* 10 */  cg.addIns(Runtime::CALL);
   /* 11 */  cg.addIns(Runtime::PUSHL,0);
   /* 12 */  cg.addIns(Runtime::PUSHI,2);
   /* 13 */  cg.addIns(Runtime::ISUB);
   /* 14 */  cg.addIns(Runtime::PUSHC,fp);
   /* 15 */  cg.addIns(Runtime::CALL);
   /* 16 */  cg.addIns(Runtime::IADD);
   /* 17 */  cg.addIns(Runtime::RET);
#endif
   cg.endFunction();
   
   const Function *fib = cg.getFunction();
   
   cout << *fib << endl;
   for ( int i=0; i<30; i++ ) {
      cout << "fib(" << i << ") evaluates to :";
      cout << fib->computeInteger(&env,i) << endl;
   }
   return 0;
}
