#include <graph/codegen.h>

using graph::CodeGen;
using graph::Function;
using graph::Runtime;

#define WIDTH 1000
#define HEIGHT 1000

double values[WIDTH][HEIGHT];

int main(int argc,char *argv[])
{
   Runtime env;
   CodeGen cg(env);
   
   // f(x,y,z) := 
   //  5 * cos(x/2) * cos(y/2) * cos(z) + sin(7 * (x + z)) / 6 + sin(9 * (x/2 + z/3 + y)) / 7
   unsigned fp = cg.beginFunction("f","(fff)f",3);
#ifdef USE_ACC
   /*  0 */  cg.addIns(Runtime::LDI,5);   // acc:5
   /*  1 */  cg.addIns(Runtime::I2F);     // acc:5.0
   /*  2 */  cg.addIns(Runtime::PUSH);    // acc:5.0; stack: 5.0

   /*  3 */  cg.addIns(Runtime::PUSHL,0);   // acc: x  ; stack: 5.0

   /*  4 */  cg.addIns(Runtime::LDI,2);   // acc: 2  ; stack: 5.0,x
   /*  5 */  cg.addIns(Runtime::I2F);     // acc: 2.0; stack: 5.0,x
   /*  6 */  cg.addIns(Runtime::FDIV);    // acc: x/2.0;stack: 5.0
   /*  7 */  cg.addIns(Runtime::COS);     // acc: cos(x/2.0);stack: 5.0
   /*  8 */  cg.addIns(Runtime::PUSH);    // acc: cos(x/2.0); stack: 5.0,cos(x/2.0)
   /*  9 */  cg.addIns(Runtime::PUSHL,1);   // acc: y  ; stack: 5.0,cos(x/2.0)

   /* 10 */  cg.addIns(Runtime::LDI,2);   // acc: 2  ; stack: 5.0,cos(x/2.0),y
   /* 11 */  cg.addIns(Runtime::I2F);     // acc: 2.0; stack: 5.0,cos(x/2.0),y
   /* 12 */  cg.addIns(Runtime::FDIV);    // acc: y/2.0; stack: 5.0,cos(x/2.0)
   /* 13 */  cg.addIns(Runtime::COS);     // acc: cos(y/2.0); stack: 5.0,cos(x/2.0)


   /* 14 */  cg.addIns(Runtime::LDL_PUSH,2);   // acc: z  ; stack: 5.0,cos(x/2.0),cos(y/2.0) 
   /* 15 */  cg.addIns(Runtime::COS);     // acc: cos(z); stack: 5.0,cos(x/2.0),cos(y/2.0) 
   /* 16 */  cg.addIns(Runtime::FMUL);    // acc: cos(y/2.0)*cos(z); stack: 5.0,cos(x/2.0)
   /* 17 */  cg.addIns(Runtime::FMUL);    // acc: cos(x/2.0)*cos(y/2.0)*cos(z); stack: 5.0
   /* 18 */  cg.addIns(Runtime::FMUL);    // acc: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z); stack:

   /* 19 */  cg.addIns(Runtime::LDI_PUSH,7);   // acc: 7; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)
   /* 20 */  cg.addIns(Runtime::I2F);     // acc: 7.0; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)
   /* 21 */  cg.addIns(Runtime::PUSH);    // acc: 7.0; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z),7.0

   /* 22 */  cg.addIns(Runtime::PUSHL,0);   // acc: x; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z),7.0

   /* 23 */  cg.addIns(Runtime::LDL,2);   // acc: z; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z),7.0,x
   /* 24 */  cg.addIns(Runtime::FADD);    // acc: x+z; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z),7.0
   /* 25 */  cg.addIns(Runtime::FMUL);    // acc: 7.0*(x+z); stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)
   /* 26 */  cg.addIns(Runtime::SIN);     // acc: sin(7.0*(x+z)); stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)

   /* 27 */  cg.addIns(Runtime::LDI_PUSH,6);   // acc: 6
   /* 28 */  cg.addIns(Runtime::I2F);     // acc: 6.0
   /* 29 */  cg.addIns(Runtime::FDIV);    // acc: sin(7.0*(x+z))/6.0; stack: 5.0*cos(x/2.0)*cos(y/2.0)*cos(z)


   /* 30 */  cg.addIns(Runtime::LDI_PUSH,9); // 9
   /* 31 */  cg.addIns(Runtime::I2F);     // 9 - 9.0
   /* 32 */  cg.addIns(Runtime::PUSH);

   /* 33 */  cg.addIns(Runtime::PUSHL,0); // x

   /* 34 */  cg.addIns(Runtime::LDI,2); // 2
   /* 35 */  cg.addIns(Runtime::I2F);     // 2 -> 2.0
   /* 36 */  cg.addIns(Runtime::FDIV);    // /
   /* 37 */  cg.addIns(Runtime::PUSH);

   /* 38 */  cg.addIns(Runtime::PUSHL,2); // z

   /* 39 */  cg.addIns(Runtime::LDI,3); // 3
   /* 40 */  cg.addIns(Runtime::I2F);     // 3 - 3.0
   /* 41 */  cg.addIns(Runtime::FDIV);    // /

   /* 42 */  cg.addIns(Runtime::LDL_PUSH,1); // y
   /* 43 */  cg.addIns(Runtime::FADD);    // +
   /* 44 */  cg.addIns(Runtime::FADD);    // +
   /* 45 */  cg.addIns(Runtime::FMUL);    // *
   /* 46 */  cg.addIns(Runtime::SIN);     // sin

   /* 47 */  cg.addIns(Runtime::LDI_PUSH,7); // 7
   /* 48 */  cg.addIns(Runtime::I2F);     // 7 -> 7.0
   /* 49 */  cg.addIns(Runtime::FDIV);    // /
   /* 50 */  cg.addIns(Runtime::FADD);    // +
   /* 51 */  cg.addIns(Runtime::FADD);    // +
   /* 52 */  cg.addIns(Runtime::RET);     // return

#else
   /*  0 */  cg.addIns(Runtime::PUSHI,5); // 5
   /*  1 */  cg.addIns(Runtime::I2F);     // 5 -> 5.0
   /*  2 */  cg.addIns(Runtime::PUSHL,0); // x
   /*  3 */  cg.addIns(Runtime::PUSHI,2); // 2
   /*  4 */  cg.addIns(Runtime::I2F);     // 2 -> 2.0
   /*  5 */  cg.addIns(Runtime::FDIV);    // /
   /*  6 */  cg.addIns(Runtime::COS);     // cos
   /*  7 */  cg.addIns(Runtime::PUSHL,1); // y
   /*  8 */  cg.addIns(Runtime::PUSHI,2); // 2
   /*  9 */  cg.addIns(Runtime::I2F);     // 2 -> 2.0
   /* 10 */  cg.addIns(Runtime::FDIV);    // /
   /* 11 */  cg.addIns(Runtime::COS);     // cos
   /* 12 */  cg.addIns(Runtime::PUSHL,2); // z
   /* 13 */  cg.addIns(Runtime::COS);     // cos
   /* 14 */  cg.addIns(Runtime::FMUL);    // *
   /* 15 */  cg.addIns(Runtime::FMUL);    // *
   /* 16 */  cg.addIns(Runtime::FMUL);    // *
   /* 17 */  cg.addIns(Runtime::PUSHI,7); // 7
   /* 18 */  cg.addIns(Runtime::I2F);     // 7 -> 7.0
   /* 19 */  cg.addIns(Runtime::PUSHL,0); // x
   /* 20 */  cg.addIns(Runtime::PUSHL,2); // z
   /* 21 */  cg.addIns(Runtime::FADD);    // +
   /* 22 */  cg.addIns(Runtime::FMUL);    // *
   /* 23 */  cg.addIns(Runtime::SIN);     // sin
   /* 24 */  cg.addIns(Runtime::PUSHI,6); // 6
   /* 25 */  cg.addIns(Runtime::I2F);     // 6 -> 6.0
   /* 26 */  cg.addIns(Runtime::FDIV);    // /
   /* 27 */  cg.addIns(Runtime::PUSHI,9); // 9
   /* 28 */  cg.addIns(Runtime::I2F);     // 9 - 9.0
   /* 29 */  cg.addIns(Runtime::PUSHL,0); // x
   /* 30 */  cg.addIns(Runtime::PUSHI,2); // 2
   /* 31 */  cg.addIns(Runtime::I2F);     // 2 -> 2.0
   /* 32 */  cg.addIns(Runtime::FDIV);    // /
   /* 33 */  cg.addIns(Runtime::PUSHL,2); // z
   /* 34 */  cg.addIns(Runtime::PUSHI,3); // 3
   /* 35 */  cg.addIns(Runtime::I2F);     // 3 - 3.0
   /* 36 */  cg.addIns(Runtime::FDIV);    // /
   /* 37 */  cg.addIns(Runtime::PUSHL,1); // y
   /* 38 */  cg.addIns(Runtime::FADD);    // +
   /* 39 */  cg.addIns(Runtime::FADD);    // +
   /* 40 */  cg.addIns(Runtime::FMUL);    // *
   /* 41 */  cg.addIns(Runtime::SIN);     // sin
   /* 42 */  cg.addIns(Runtime::PUSHI,7); // 7
   /* 43 */  cg.addIns(Runtime::I2F);     // 7 -> 7.0
   /* 44 */  cg.addIns(Runtime::FDIV);    // /
   /* 45 */  cg.addIns(Runtime::FADD);    // +
   /* 46 */  cg.addIns(Runtime::FADD);    // +
   /* 47 */  cg.addIns(Runtime::RET);     // return
#endif
   cg.endFunction();
   
   const Function *f = cg.getFunction();
   
   cout << *f << endl;

   for ( int x=0; x<WIDTH; x++ ) {
      for ( int y=0; y<HEIGHT; y++ ) {
         values[x][y] = f->computeReal(&env,(double)x,(double)y,1.0);
      }
   }

   if ( argc == 2 && !strcmp(argv[1],"-v") ) {
      for ( int x=0; x<WIDTH; x++ ) {
         for ( int y=0; y<HEIGHT; y++ ) {
            cout << "f(" << x << ',' << y << ',' << 1.0 << ") evaluates to :";
            cout << values[x][y] << endl;
         }
      }
   }

   return 0;
}
